.\" bm# 0000002139
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)acctcom.C	1.1);


.ds mc 
.ds rv Revision 1
.TH ACCTCOM C
.SH Name
acctcom \- Searches for and prints process accounting files.
.SH Syntax
.B acctcom
[\^[\^options\^]\^[\^file\^]\^] . . .
.SH Description
.I Acctcom\^
reads
.IR file ,
the standard input, or
.BR /usr/adm/pacct ,
in the form described by
.IR acct (F)
and writes selected records to the standard output.
Each record represents the execution of one process.
The output shows the
\s-1COMMAND\s0
\s-1NAME\s0,
\s-1USER\s0,
\s-1TTYNAME\s0,
\s-1START\s0
\s-1TIME\s0,
\s-1END\s0
\s-1TIME\s0,
\s-1REAL\s0
(\s-1SEC\s0),
\s-1CPU\s0
(\s-1SEC\s0),
\s-1MEAN\s0
\s-1SIZE\s0\|(\s-1K\s0),
and optionally,
\s-1F\s0
(the
.IR fork / exec
flag:
\fB1\fP for \fIfork\fP without \fIexec\fP)
and
\s-1STAT\s0
(the system exit status).
.PP
The command name is prepended with a #
if it was executed with
super-user
privileges.
If a process is not associated with a known terminal,
a \fB?\fP is printed in the \s-1TTYNAME\s0
field.
.PP
If no \fIfiles\fP are specified, and
if the standard input is associated with a terminal
or
.B /dev/null
(as is the case when using
.B &
in the shell),
.B /usr/adm/pacct
is read, otherwise the standard input is read.
.PP
If any
.I file\^
arguments are given,
they are read in their respective order.
Each file is normally read forward, i.e., in chronological order
by process completion time.
The file
.B /usr/adm/pacct
is usually the current file to be examined;
a busy system may need several files,
in which case all but the current file will be found in
.BR /usr/adm/pacct\fI?\fP .
The \fIoptions\fP are:
.PP
.PD 1 '\"was0
.TP 10 '\"was12
.B \-b
Reads backwards, showing latest commands first.
.TP
.B \-f
Prints the
.IR fork / exec
flag and system exit status columns in the output.
.TP
.B \-h
Instead of mean memory size,
shows the fraction of total available
.SM CPU
time consumed by the process
during its execution.
This ``hog factor'' is computed as:
.sp 10p '\"\nPDu
.RS 10m
(total\ \s-1CPU\s+1\ time)/(elapsed\ time).
.RE
.TP
.B \-i
Prints columns containing the \s-1I\s0/\s-1O\s0 counts in the output.
.TP
.B \-k
Instead of memory size, shows total kcore-minutes.
.TP
.B \-m
Shows mean core size (the default).
.TP
.B \-r
Shows
.SM CPU
factor (user time/(system-time + user-time).)
.TP
.B \-t
Shows separate system and user
.SM CPU
times.
.TP
.B \-v
Excludes column headings from the output.
.TP
.BI \-l \ line\^
Shows only processes belonging to terminal
.BI /dev/ line\^\fR.\fP
.TP
.BI \-u \ user\^
Shows only processes belonging to
.I user\^
that may be specified by a user \s-1ID\s0,
a login name
that is then converted to a user \s-1ID\s0,
a \fB#\fP which designates only those processes executed with
super-user
privileges, or
.B ?
which designates only those processes associated with unknown
user \s-1ID\s0s.
.TP
.BI \-g \ group\^
Shows only processes belonging to
.IR group .
The \fIgroup\fP may be designated by either the
group \s-1ID\s+1 or group name.
.TP
.BI \-d \ mm/dd\^
Any
.IR time\^
arguments following this flag are assumed to occur on
the given month and day,
rather than during the last 24 hours.
This is needed for looking at old files.
.TP
.BI \-s \ time\^
Shows only those processes that existed on or after
.IR time ,
given in the form
.IB hr : min : sec \fR.\fP
The
.BI : sec\^
or
.BI : min : sec\^
may be omitted.
.TP
.BI \-e \ time\^
Shows only those processes that existed on or before
.IR time .
Using the same
.I time\^
for both
.B \-s
and
.B \-e
shows the processes that existed at
.IR time .
.TP
.BI \-n \ pattern\^
Shows only commands matching
.I pattern\^
that may be a regular expression as in
.IR ed\| (C)
except that \fB+\fP means one or more occurrences.
.TP
.BI \-H \ factor\^
Shows only processes that exceed
.IR factor ,
where factor is the ``hog factor'' as explained in option
.BR \-h \ above.
.TP
.BI \-O \ time\^
Shows only those processes
with operating system \s-1CPU\s0 time
that exceeds
.IR time .
.TP
.BI \-C \ time\^
Shows only those processes that exceed
.I time\^
(the total \s-1CPU\s0 time).
.PD
.PP
Multiple options have the effect of a logical
.SM AND.
.bp
.SH Files
/etc/passwd
.br
.sp 10p
/usr/adm/pacct
.br
.sp 10p
/etc/group
.SH "See Also"
accton(C),
ps(C),
su(C),
acct(S),
acct(F),
utmp(M)
.SH Notes
.I Acctcom\^
only reports on processes that have terminated; use
.IR ps\^ (C)
for active processes.
.br
.DT
