.\" bm# 0000002134
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)awk.C	1.1);


.ds mc 
.ds rv Revision 1
.TH AWK C
.SH Name
awk \- Searches for and processes a pattern in a file.
.SH Syntax
.B awk
[
.BR \-F \fIc\fR
] [ \-f \fIprogramfile\fR | '\fIprogram\fR' ] [ \fIfile\fR ... ]
.SH Description
.I Awk
scans each input 
.I file 
for lines that match patterns specified in
.I program
or in 
.IR programfile .
When a line of \fIfile\fR matches a pattern, an
associated action may be performed. 
.I Awk
is useful for compiling information,
performing arithmetic on input data, and for doing
iterative or conditional processing.
.P
The options are:
.TP 9
.B \-F\fIc\fR
Sets the field separator variable (FS) to the letter
\&``c''. The default field separators are tab and space.
.TP
.B \-f
Causes 
.I awk
to take its program from 
.IR programfile .
.P
The arguments are:
.TP 9
.I programfile
A file containing an \fIawk\fR program.
.TP
.I program
An 
.I awk
program. Programs given on the command line must be enclosed 
in single quotation marks to prevent interpretation by the
shell.
.TP
.I file\^\fR ...
The name(s) of the file or files to be processed. If no
filename is given, the standard output is used. 
.P
An
.I awk
program consists of statements in the form:
.IP
pattern { action }
.P
Pattern-action statements may appear on the 
.I awk
command line, or in an 
.I awk 
program file. 
.P
If no \fIpattern\fR is given, all lines in the input file
are matched. If no \fIaction\fR is given, each matched
line is displayed on the standard output.
.P 
A pattern may be a literal string or a regular
expression, or a combination of a regular expression 
and a field or variable separated by operators. 
.P
.I Awk
also provides two patterns,
.SM BEGIN
and 
.SM END,
that can be used to perform actions before the first
line is read, and after the last line is read,
respectively.
.P
To select a range of lines, use two patterns on a single
program line, separated by a comma. 
.P
An action is a sequence of statements separated by a
semicolon, newline, or right brace. See 
\fIStatements\fR later in this section.
.SS \fIVariables\fR
.P
In addition to variables declared and initialized by the
user, \fIawk\fR has the following program variables:
.TP 10
NR
Number of records.
.TP
NF
Number of fields in a record.
.TP
FS
Input field separator.
.TP
OFS
Output field separator.
.TP
RS
Input record separator.
.TP
ORS
Output record separator.
.TP 
$0
The current record.
.TP
$1, $\fIn\fR
Fields in the current record.
.TP
OFM
The ouput format for numbers. The default
is %.6g.
.TP
FILENAME
The name of the input file currently being read.
.P
Arrays may be used to store data. Arrays do not need to be
dimensioned before use. For example, w[i] denotes the 
\fIi\fRth item of array \fIw\fR.
.SS \fIExpressions\fR
.P
A pattern match with a field or variable may be
tested with the following operators:
.tr ~~
.TP
~ 
Matches the regular expression.
.TP
!~
Does not match the regular expression.
.ne 1i
.P
.I Awk
processes relational expressions using the following
operators:
.TP 9
<
Less than
.TP
<= 
Less than or equal to
.TP
=\^=
Equal to
.TP
!=
Not equal to
.TP
>=
Greater than or equal to
.TP
>
Greater than
.P
Patterns can be combined using
the operators:
.TP 9
&&
And
.TP
||
Or
.TP
!
Not
.P
An empty expression-list stands for the whole line.
Expressions take on string or numeric values as appropriate,
and are built using the following operators:
.TP 9
.B + 
Addition
.TP
.B \- 
Subtraction
.TP
.B \(** 
Multiplication
.TP
.B / 
Division
.TP
.B % 
Modulo
.P
Concatenation is indicated by a blank.
.P
The following C operators
are also available in expressions:
.TP 9
.B ++ 
Increment
.TP
.B \-\ \- 
Decrement
.TP
.B += 
Add and assign
.TP
.B \-=
Subtract and assign
.TP
.B \(**= 
Multiply and assign
.TP
.BR /= 
Divide and assign
.TP
.B %=
Modulo and assign
.SS \fIStatements\fR
.PP
.ss 18
.RS
.nf
if ( \fIconditional\fR ) \fIstatement\fR [ else \fIstatement\fR ]
while ( \fIconditional\fR ) \fIstatement\fR
for ( \fIexpression\fR ; \fIconditional\fR ; \fIexpression\fR ) \fIstatement\fR
break
continue
{ [ \fIstatement\fR ] .\|.\|. }
\fIvariable\fR = \fIexpression\fR
print [ \fIexpression-list\fR ] [ >\fIexpression\fR ]
printf \fIformat\fR [ , \fIexpression-list\fR ] [ >\fIexpression\fR ]
next 	#skip remaining patterns on input line
.fi
.RE
.ss 12
.TP 9
while
Used the same as in C.
.TP
for
The iterative construction. It can be used the same as in
the C language, or as an array iterator.
.TP
break
Similar to its C counterpart.
.TP
continue
Similar to its C counterpart.
.TP
print
Prints its arguments on the standard output, or in a file
if redirected.
.TP
printf
Prints 
.I expression-list
in the format specified in 
.I format. 
See \fIprintf\^\fR(S).
.TP
next
Stops processing the current record and moves to the
next record, if any.
.PP
Comments are preceded by a number sign (#).
.SS \fIFunctions\fR
.P
.I Awk
has the following built-in functions:
.TP 8
exit(\fIx\fR)
Terminates the 
.I awk
program. If \fIx\fR is given, this value is \fIawk\fR's
return value. If \fIx\fR is not given, 0 is returned.
If the program has an 
.SM END
section, it is invoked before termination.
.TP
exp(\fIx\fR)
Exponentiation of the value of \fIx\fR.
.TP
index(\fIs, t\fR)
Returns the starting position of the
leftmost occurrence of \fIt\fR in \fIs\fR. If \fIt\fR is
not a substring of \fIs\fR, then index(\fIs, t\fR) is 0.
.TP
int(\fIx\fR)
Returns the largest integer less than
or equal to \fIx\fR. If \fIx\fR is negative, its value is
the smallest integer greater than or equal to \fIx\fR.
.TP 8
length(\fIx\fR)
A function whose value is the number of characters in
the string (\fIx\fR). With no arguments \fIlength\fR is
equivalent to $0.
.TP
log(\fIx\fR)
Natural logarithm of \fIx\fR.
.TP
split(\fIx, y\fR)
Assigns the fields of string \fIx\fR to successive
elements of array \fIy\fR.
.TP
sqrt(\fIx\fR)
Square root of \fIx\fR.
.TP
substr(\fIstring, index, length\fR)
Returns the substring of \fIstring\fR that begins at
\fIindex\fR and is \fIlength\fR characters long.
.SH Examples
The following displays lines in \fIfile\fR longer than 72 characters:
.PP
.ss 18
.RS
awk '{length > 72}' \fIfile\fR
.RE
.ss 12
.PP
The following prints the first two fields in opposite order:
.PP
.ss 18
.RS
awk '{ print $2, $1 }' \fIfile\fR
.RE
.ss 12
.PP
The following adds up the first columns and prints their 
sum and average:
.PP
.ss 18
.RS
	{ s += $1 }
.br
\s-1END\s+1	{ print "sum is", s, " average is", s/\s-1NR\s+1 }
.RE
.ss 12
.PP
The following prints the  fields in 
.I file
in reverse order:
.PP
.ss 18
.RS
awk { for (i = \s-1NF\s+1; i > 0; \-\^-i) print $i } \fIfile\fR
.RE
.ss 12
.PP
The following 
.I awk
program file will print all lines in the object file whose first field 
is different from the first field in the previous line:
.PP
.ss 18
.RS
$1 != prev { print; prev = $1 }
.RE
.ss 12
.SH "See Also"
grep(C), lex(CP), sed(C)
.br
.I "The XENIX Text Processing Guide"
'\".I "Awk\-A Pattern Scanning and Processing Language\^"
'\"by A. V. Aho, B. W. Kernighan, and P. J. Weinberger.
.SH Notes
Input whitespace is not preserved on output if fields are involved.
.PP
There are no explicit conversions between numbers and strings.
To force an expression to be treated as a number add 0 to it;
to force it to be treated as a string concatenate the
null string (\f3"\^"\fP) to it.
.PP
This command is more fully documented in the 
.I "XENIX Text Processing Guide."
.tr ``
