.\" bm# 0000002131
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)bc.C	1.1);


.ds mc 
.ds rv Revision 1
.TH BC C 
.SH Name
bc \- Invokes a calculator.
.SH Syntax
.B bc
[
.B \-c
] [
.B \-l
] [ file ... ]
.SH Description
.I Bc\^
is an interactive processor for a language that resembles
C but provides unlimited precision arithmetic.
It takes input from any files given, then reads
the standard input.
The
.B \-l
argument stands for the name
of an arbitrary precision math library.
The syntax for 
.I bc\^
programs is as follows:
L means the letters a\-z,
E means expression, S means statement.
.HP 4 '\"was 6
Comments:
.br
.sp 10p
Enclosed in
.B /\(**
and
.BR \(**/ 
.HP 4 '\"6
Names:
.br
.sp 10p
Simple variables: L
.br
Array elements: L [ E ]
.br
The words ``ibase'', ``obase'', and ``scale''
.HP 4 ''\" was 6
Other operands:
.br
.sp 10p
Arbitrarily long numbers with optional sign and decimal point
.br
( E )
.br
sqrt ( E )
.br
length ( E )	Number of significant decimal digits
.br
scale ( E )	Number of digits right of decimal point
.br
L ( E , ... , E )
.HP 4
Additive operators:
.br
.sp 10p
.B 
+ 
.br
\- 
.HP 4
Multiplicative operators:
.br
.sp 10p
.B 
.br
\(**
.br
/ 
.br
% (remainder) 
.br
^ (exponentiation)
.bp
.HP 4
Unary operators:
.sp 10p
.B
++ 
.br
\-\ \-	(prefix and postfix; apply to names)
.HP 4
Relational operators:
.sp 10p
.B 
== 
.br
<=
.br
>= 
.br
!=  
.br
< 
.br
>
.HP 4
Assignment operators:
.br
.sp 10p
.B
=
.br
=+
.br
=\-
.br
=\(** 
.br
=/
.br
=%
.br
=^
.HP 4
Statements:
.br
.sp 10p
E
.br
{ S ; ... ; S }
.br
if ( E ) S
.br
while ( E ) S
.br
for ( E ; E ; E ) S
.br
null statement
.br
break
.br
quit
.HP 4
Function definitions:
.br
.sp 10p
define L ( L ,..., L ) {
.br
	auto L, ... , L
.br
	S; ... S
.br
	return ( E )
.br
}
.bp
.HP 4
Functions in 
.B \-l
math library:
.br
.sp 10p
s(x)	Sine
.br
c(x)	Cosine
.br
e(x)	Exponential
.br
l(x)	Log
.br
a(x)	Arctangent
.br
j(n,x)	Bessel function
.PP
.DT
All function arguments are passed by value.
.PP
The value of a statement that is an expression is printed
unless the main operator is an assignment.
Either semicolons or newlines may separate statements.
Assignment to
.I scale\^
influences the number of digits to be retained on arithmetic
operations in the manner of
.IR dc (C).
Assignments to
.I ibase\^
or
.I obase\^
set the input and output number radix respectively.
.PP
The same letter may be used as an array, a function,
and a simple variable simultaneously.
All variables are global to the program.
``Auto'' variables are pushed down during function calls.
When using arrays as function arguments
or defining them as automatic variables,
empty square brackets must follow the array name.
.PP
.I Bc\^
is actually a preprocessor for
.IR dc (C),
which it invokes automatically, unless the
.B \-c
(compile only)
option is present.
If the 
.B \-c
option is present, the
.I dc\^
input is sent to the standard output instead.
.SH Example
.PP
.fi
The following
defines a function to compute an approximate value of
the exponential function: 
.PP
.nf
	scale = 20
	define e(x){
		auto a, b, c, i, s
		a = 1
		b = 1
		s = 1
		for(i=1; 1==1; i++){
			a = a\(**x
			b = b\(**i
			c = a/b
			if(c == 0) return(s)
			s = s+c
		}
	}
.fi
.PP
.ne 1i
The following prints the approximate values of
the exponential function of the first ten
integers:
.PP
.nf
	for(i=1; i<=10; i++) e(i)
.fi
.PP
.SH Files
/usr/lib/lib.bc	Mathematical library
.sp 10p
/usr/bin/dc	Desk calculator proper
.SH "See Also"
dc(C)
.br
.I "The XENIX User's Guide"
'\".I "BC \- An Arbitrary Precision Desk-Calculator Language\^"
'\" by L. L. Cherry and R. Morris.
.SH Notes
'\"	No \f3&&\fP, gp\|| yet.
'\"	.br
'\"	.sp 10p
A
.I For\^
statement must have all three E's.
.br
.sp 10p
.I Quit\^
is interpreted when read, not when executed.
