.\" bm# 0000002129
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)bfs.C	1.1);


.ds mc 
.ds rv Revision 1
.TH BFS C
.SH Name
bfs \- Scans big files.
.SH Syntax
.B bfs
[
.B \-
] name
.SH Description
.I Bfs\^
is like
.IR ed\| (C)
except that it is read-only and processes much larger files.
Files can be up to 1024K bytes and
32K lines, 
with up to 255 characters per line.
.I Bfs\^
is usually more efficient than
.I ed\^
for scanning a file,
since the file is not copied to a buffer.
It is most useful for identifying sections of
a large file where
.IR csplit\| (C)
can be used to divide it into more manageable pieces for editing.
.PP
Normally, the size of the file being scanned is printed,
as is the size of any file written with the
.B w
command.
The optional dash
.RB ( \- )
suppresses printing of sizes.
Input is prompted for with an asterisk
.RB ( \(** )
by default.
If a ``P''
and a 
.SM RETURN 
are typed as in
.IR ed ,
then prompting is turned off.
The ``P'' acts as a toggle, so
prompting can be turned on again by
entering another ``P'' and a 
.SM RETURN.
Note that messages are given in response
to errors only if prompting is turned on.
.PP
All address expressions described under
.I ed\^
are supported.
In addition, regular expressions may be surrounded with two
symbols other than the standard slash
.RB  (\^ / \^)
and
.RB ( ? ):
A greater-than sign 
.RB ( > )
indicates downward search without wraparound,
and a less-than sign
.RB  ( < )
indicates upward search without wraparound.
Since
.I bfs\^
uses a different regular expression-matching routine from 
.IR ed ,
the regular expressions accepted are slightly wider 
in scope (see
.IR regex\| (S)).
The differences from 
.I ed
syntax include the fact that parentheses and curly
braces are special and need not be escaped.
Differences are listed below:
.TP "\w'\fB(.\|.\|.\^)$n\fR\ \ \ 'u"
.B +
A regular expression followed by \fB+\fP means
.IR "one or more times" .
For example,
.B [0\-9]+
is equivalent to
.BR [0\-9][0\-9]\(** .
.TP
.B "{m} {m,} {m,u}"
Integer values enclosed in \fB{\|}\fP indicate the
number of times the preceding regular expression is to be applied.
.I m\^
is the minimum number and
.I u\^
is a number, less than 256, which is the maximum.
If only
.I m\^
is present (e.g., {m}),
it indicates the exact number of times the regular
expression is to be applied.
{m,} is analogous to {m,infinity}.
The plus (\fB+\fP) and star (\fB\(**\fP) operations are
equivalent to {1,} and {0,} respectively.
.TP
.B "( .\|.\|. )$\fIn\^\fP"
The value of the enclosed regular expression is
to be returned.
The value
will be stored in the
.IR (n+1) th
argument following the subject argument.
At most ten enclosed regular expressions are allowed.
.I Regex\^
makes its assignments unconditionally.
.TP
.B "( .\|.\|. )"
Parentheses are used for grouping.
An operator, e.g.
.BR \(** ", " + ", " {\|} ,
can work on a single character or a regular
expression enclosed in parenthesis.
For example, (a\(**(cb+)\(**)$0.
.PP
There is also a slight difference in mark names: 
only the letters
``a'' through ``z'' may be used,
and all 26 marks are remembered.
.PP
The
.BR e ,
.BR g ,
.BR v ,
.BR k ,
.BR n ,
.BR p ,
.BR q ,
.BR w ,
.BR = ,
.B !
and null commands operate as described under
.IR ed .
Commands such as \fB-\^-\^-\fP, \fB+++\-\fP,
\fB+++=\fP, \fB\-12\fP, and \fB+4p\fP are accepted.
Note that \fB1,10p\fP and \fB1,10\fP will both print 
the first ten lines.
The
.B f
command only prints the name of the file being scanned;
there is no remembered filename.
The
.B w
command is independent of output diversion,
truncation, or crunching
(see the
.BR xo , \ xt \ and \ xc
commands, below).
The following additional commands are available:
.TP 5 
.BI xf " file"\^
Further commands are taken from the named
.IR file .
When an end-of-file is reached,
an interrupt signal is received, or an error occurs,
reading resumes with the
file containing the
.BR xf .
.B \s-1X\s0f
commands may be nested to a depth of 10.
.TP 
.BI xo " \fR[\|\fPfile\fR\|]\fP"\^
Further output from the
.B p
and null
commands is diverted to the named
.I file.
If
.I file\^
is missing, output is diverted to the standard output.
Note that each diversion causes truncation
or creation of the file.
.TP 
.BI : " label"\^
This positions a
.I label\^
in a command file.
The
.I label\^
is terminated by a newline, and
blanks between the
.B :
and the start of the
.I label\^
are ignored.
This command may also be used to insert comments
into a command file,
since labels need not be referenced.
.TP 
( \fB. \fR, \fB. \fR)\fBxb\fP/\fIregular expression\fP/\fIlabel\fR
A jump (either upward or downward) is made to \fIlabel\fP if
the command succeeds.
It fails under any of the following conditions:
.IP
1.\0\0Either address is not between
.B 1
and
.BR $ .
.IP
2.\0\0The second address is less than the first.
.IP
3.\0\0The regular expression doesn't match at least one line
.br
\0\0\0\0in the specified range, including the first and last lines.
.IP
On success, dot 
.RB ( \fB.\fP )
is set to the line matched and a jump
is made to 
.I label.
This command is the only one that doesn't issue an error
message on bad addresses, so it may be used to
test whether addresses are bad before other commands are executed.
Note that the command
.RS 10
.sp
xb/^/ label
.RE
.IP
is an unconditional jump.
.br
.sp 10p
The
.B xb
command is allowed only if
it is read from somewhere other than a terminal.
If it is read from a pipe only a downward jump is possible.
.TP
.BI xt " number"\^
Output from the
.B p
and null commands is
truncated to a maximum of
.I number\^
characters.
The initial number is 255.
.TP
\fBxv\fR[\^\fIdigit\fR\^]\|[\^\fIspaces\fR\^]\|[\^\fIvalue\fR\^]
The variable name is the specified
.I digit\^
following the \fBxv\fP.
\fBXv5100\fP or \fBxv5 100\fP both
assign the value 
\fB100\fP to the variable
.BR 5 .
.B \s-1X\s0v61,100p
assigns the value
.B 1,100p
to the variable
.BR 6 .
To reference a variable, put a
.B %
in front of the variable name.
For example, using the above assignments
for variables \fB5\fP and \fB6\fP:
.RS 10
.sp
1,%5p
.br 
1,%5
.br 
%6
.br
.RE
.TP
\&
prints the first 100 lines.
.RS 10
.sp
g/%5/p
.RE
.TP
\&
globally searches for the characters \fB100\fP
and prints each line containing a match.
To escape the special meaning of
.BR % ,
a
.B \e
must
precede it.
For example,
.RS 10
.sp
g/".*\\%[cds]/p
.RE
.TP
\&
could be used to match and list lines 
containing
.I printf\^
characters, decimal integers, or strings.
.br
.sp 10p
Another feature of the
.B xv
command is that the first line
of output from a \s-1XENIX\s0 command can
be stored into a variable.
The only
requirement is that the first character
of
.I value\^
be a
\fB!\fP.
For example,
.RS 10
.sp
xv5!cat junk
.br 
!rm junk
.br 
!echo "%5"
.br 
xv6!expr %6 + 1
.RE
.TP
\&
puts the current line in variable \fB5\fP,
prints it, and increments the variable \fB6\fP by one.
To escape the special meaning of
.B !
as the
first character of
.IR value ,
precede it with a
\fB\e\fP.
For example,
.RS 10
.sp
xv7\\!date
.RE
.TP
\&
stores the
value \fB!date\fP into
variable \fB7\fP.
.TP
.BI xbz " label"\^
\&
.PD 0
.TP
.BI xbn " label"\^
.PD
These two commands test the last saved
.I "return code\^"
from the execution of a
.SM XENIX
command
(\fB!\fIcommand\^\fR)
or nonzero value, respectively, and jump to the
specified label.
The two examples below 
search for the next five lines containing
the string \fBsize\fP:
.RS 10
.sp
xv55
.br 
: l
.br 
/size/
.br 
xv5!expr %5 \- 1
.br 
!if 0%5 != 0 exit 2
.br 
xbn l
.sp
xv45
.br 
: l
.br 
/size/
.br 
xv4!expr %4 \- 1
.br 
!if 0%4 = 0 exit 2
.br 
xbz l
.br
.RE
.TP
\fBxc\fP\ [\fIswitch\fP\^]
If
.I switch\^
is \fB1\fP, output from the
.B p
and null commands is crunched;
if
.I switch\^
is \fB0\fP it isn't.
Without an argument,
.B xc
reverses 
.I switch.
Initially, 
.I switch
is set for no crunching.
Crunched output has strings of tabs and blanks reduced
to one blank and blank lines suppressed.
.RE
.PP
.i0
.fi
.i0
.SH "See Also"
csplit(C), ed(C), regex(S)
.SH Diagnostics 
If prompting is turned off, a question mark is printed 
.RB ( ? )
for errors in commands. When prompting is on,
self-explanatory error messages appear. 
