.\" bm# 0000002123
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)chmod.C	1.1);


.ds mc 
.ds rv Revision 1
.TH CHMOD C
.SH Name
chmod \- Changes the access permissions of a file or directory.
.SH Syntax
.B chmod
\fImode file\fI ...
.SH Description
The
.I chmod
command changes the access permissions (or \fImode\fR) of
a specified file or directory. It is used to control file
and directory access by users other than the owner and
super-user.  The \fImode\fR may be an expression
composed of letters and operators (called \fIsymbolic
mode\fR), or a number (called \fIabsolute mode\fR).
.P
A 
.I chmod 
command using \fIsymbolic mode\fR has the form:
.IP
chmod [\fIwho\fR] +\^\-\^= [permission ...] \fIfilename\fR
.P
.I Who
is one or any combination of the following letters:
.TP
.B a
Stands for ``all users''. If \fIwho\fR is not indicated on
the command line, \fBa\fR is the default. The definition
of ``all users'' depends on the user's 
.I umask.
See 
\fIumask\^\fR(C).
.TP
.B g
Stands for ``group'', all users who have the
same group ID as the owner of the file or directory.
.TP
.B o
Stands for ``others'', all users on the system.
.TP 
.B u
Stands for ``user'', the owner of the file or directory.
.P
The operators are:
.TP
.B +
Adds permission
.TP
.B \-
Removes permission
.TP
.B =
Assigns the indicated permission and removes all other
permissions (if any) for that \fIwho\fR. If no permission
is assigned, existing permissions are removed.
.P
Permissions can be any combination of the following
letters:
.TP
.B x
Execute (search permission for directories)
.TP
.B r
Read
.TP
.B w
Write
.TP
.B s
Sets owner or group ID on execution of the file
to that of the owner of the file. This permission is only useful
with \fBu\fR or \fBg\fR.
.TP
.B t
Saves text in memory upon execution. (``Sticky bit'', see
\fIchmod\fR(S)). Can only be set by the super-user.
.P
Multiple symbolic modes may be given, separated by commas,
on a single command line. See the following Examples section
for sample permission settings.
.P
A 
.I chmod
command using 
.I absolute mode
has the form:
.IP
chmod \fImode\fR \fIfilename\fR
.P
where 
.I mode
is an octal number constructed by performing logical OR
on the following:
.TP 10
4000
Set user
.SM ID
on execution
.TP 10
2000
Set group
.SM ID
on execution
.TP 10
1000
Sets the sticky bit (see
.IR  chmod (S))
.TP 10
0400
Read by owner
.TP 10
0200
Write by owner
.TP 10
0100
Execute (search in directory) by owner
.TP 10
0040
Read by group
.TP
0020
Write by group
.TP
0010
Execute (search in directory) by group
.TP
0004
Read by others
.TP
0002
Write by others
.TP
0001
Execute (search in directory) by others
.TP
0000
No permissions
.SH Examples
.SS \fISymbolic Mode\fR
.P
The following command gives all users execute permission
for \fIfile\fR:
.IP
chmod +x \fIfile\fR
.P
The following command removes read and write permission
for group and others from \fIfile\fR:
.IP
chmod go-rw \fIfile\fR
.P
The following command gives other users read and write
permission for \fIfile\fR:
.IP
chmod o+rw \fIfile\fR
.P
The following commmand gives read permission to group and
other:
.IP
chmod g+r,o+r \fIfile\fR
.SS \fIAbsolute Mode\fR
.P
The following command gives all users read, write and 
execute permission for \fIfile\fR:
.IP
chmod 0777 \fIfile\fR
.P
The following command gives read and write permission to all users
for \fIfile\fR:
.IP
chmod 0666 \fIfile\fR
.P
The following command gives read and write permission to
the owner of \fIfile\fR only:
.IP
chmod 0600 \fIfile\fR
.SH "See Also"
ls(C), chmod(S)
.SH "Notes"
The user's
.I umask
may affect the default settings. 
.P
The user ID, group ID and sticky bit settings are only
useful for binary executable files. They have no effect
on shell scripts.
