.\" bm# 0000002110
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)dc.C	1.1);


.ds mc 
.ds rv Revision 1
.TH DC C 
.SH Name 
dc \- Invokes an arbitrary precision calculator.
.SH Syntax
.B dc
[ file ]
.SH Description
.I Dc\^
is an arbitrary precision arithmetic package.
Ordinarily it operates on decimal integers,
but you may specify an input base, output base,
and a number of fractional digits to be maintained.
The overall structure of
.I dc\^
is a stacking (reverse Polish) calculator.
If an argument is given, input is taken from that file until its end,
then from the standard input.
The following constructions are recognized:
.HP 6
.I number\^
.br
The value of the number is pushed on the stack.
A number is an unbroken string of the digits 0\-9.
It may be preceded by an underscore (\f3_\fP) to input a
negative number.
Numbers may contain decimal points.
.HP 6
.B "+ \- / \(** % ^"
.br
The
top two values on the stack are added
.RB ( + ),
subtracted
.RB ( \- ),
multiplied (\f3\(**\fP),
divided (\f3/\fP),
remaindered (\f3%\fP),
or exponentiated (\f3^\fP).
The two entries are popped off the stack;
the result is pushed on the stack in their place.
Any fractional part of an exponent is ignored.
.TP
.BI s x\^
The
top of the stack is popped and stored into
a register named
.IR x ,
where
.I x\^
may be any character.
If
the
.B s
is capitalized,
.I x\^
is treated as a stack and the value is pushed on it.
.TP
.BI l x\^
The
value in register
.I x\^
is pushed on the stack.
The register
.I x\^
is not altered.
All registers start with zero value.
If the
.B l
is capitalized,
register
.I x\^
is treated as a stack and its top value is popped onto the main stack.
.TP
.B  d
The
top value on the stack is duplicated.
.TP
.B  p
The top value on the stack is printed.
The top value remains unchanged.
.B p
interprets the top of the stack as an \s-1ASCII\s0 string,
removes it, and prints it.
.TP
.B  f
All values on the stack are printed.
.TP
.B  q
Exits the program.
If executing a string, the recursion level is
popped by two.
If
.B q
is capitalized,
the top value on the stack is popped 
and the string execution level is popped
by that value.
.TP
.B  x
Treats the top element of the stack as a character string
and executes it as a string of
.I dc\^
commands.
.TP
.B  X
Replaces the number on the top of the stack with its scale factor.
.TP
.B  "[ ... ]"
Puts the bracketed \s-1ASCII\s0 string onto the top of the stack.
.HP 6
.BI < "x   " > "x   " = "x   \^"
.br
The
top two elements of the stack are popped and compared.
Register
.I x\^
is evaluated if they obey the stated
relation.
.TP
.B  v
Replaces the top element on the stack by its square root.
Any existing fractional part of the argument is taken
into account, but otherwise the scale factor is ignored.
.TP
.B  !
Interprets the rest of the line as a \s-1XENIX\s0 command.
.TP
.B  c
All values on the stack are popped.
.TP
.B  i
The top value on the stack is popped and used as the
number radix for further input.
.TP
.B I
Pushes the input base on the top of the stack.
.TP
.B  o
The top value on the stack is popped and used as the
number radix for further output.
.TP
.SM
.B O
Pushes the output base on the top of the stack.
.TP
.B  k
The top of the stack is popped, and that value is used as
a nonnegative scale factor;
the appropriate number of places
are printed on output,
and maintained during multiplication, division, and exponentiation.
The interaction of scale factor,
input base, and output base will be reasonable if all are changed
together.
.TP
.B  z
The stack level is pushed onto the stack.
.TP
.SM
.B  Z
Replaces the number on the top of the stack with its length.
.TP
.B  ?
A line of input is taken from the input source (usually the terminal)
and executed.
.TP
.B "; :"
Used by 
.I bc\^
for array operations.
.bp
.SH Example
This example prints the first ten values of n!:
.nf
.PP
.in +3
[la1+dsa\(**pla10>y]sy
.br
0sa1
.br
lyx
.fi
.SH "See Also"
bc(C)
.SH Diagnostics
.TP "\w' /x is unimplemented/ \ \ \ 'u"
.I "x is unimplemented\^"
The octal number 
.I x\^
corresponds to a character that is not
implemented as a command
.TP
.I "stack empty\^"
Not enough elements on the stack to do what was asked
.TP
.I "Out of space\^"
The free list is exhausted (too many digits)
.TP
.I "Out of headers\^"
Too many numbers being kept around
.TP
.I "Out of pushdown\^"
Too many items on the stack
.TP
.I "Nesting Depth\^"
Too many levels of nested execution
.SH Notes
.I Bc
is a preprocessor for
.I dc\^,
providing infix notation and a C-like syntax
which implements functions and reasonable control
structures for programs.
For interactive use, 
.I bc
is preferred to
.I dc .
