.\" bm# 0000002093
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)expr.C	1.1);


.ds mc 
.ds rv Revision 1
.TH EXPR C
.SH Name
expr \- Evaluates arguments as an expression.
.SH Syntax
.B expr
arguments
.SH Description
.f t .ds ' \h@.05m@\s+4\v@.333m@\'\v@-.333m@\s-4\h@.05m@
.if n .ds ' '
.if t .ds ` \h@.05m@\s+4\v@.333m@\`\v@-.333m@\s-4\h@.05m@
.if n .ds ` `
The arguments are taken as an expression.
After evaluation, the result is written on the standard output.
Terms of the expression must be separated by blanks.
Characters special to the shell
must be escaped.
Note that zero is returned to indicate a zero value,
rather than the null string.
Strings containing blanks or other special characters should be quoted.
Integer-valued arguments may be preceded by a unary minus sign.
Internally, integers are treated as 32-bit, 2's complement numbers.
.PP
The operators and keywords are listed below.
Expressions should be quoted by the shell, since
many of the characters that have special meaning in
the shell also have special meaning in
.I expr.
The list is in order of increasing precedence,
with equal precedence operators grouped within 
braces (\|{ and \|}).
.TP .5i
\fIexpr \fB\^|\^\fI expr\fR
Returns the first
.I expr\^
if it is neither null nor \fB0\fP, otherwise
returns the second
.IR expr .
.TP .5i
\fIexpr \fB&\fI expr\fR
Returns the first
.I expr\^
if neither
.I expr\^
is null nor \fB0\fP, otherwise returns \fB0\fP.
.TP .5i
\fIexpr \fR{ \fB=\fR, \fB>\fR, \fB>=\fR, \fB<\fR, \fB<=\fR, \fB!=\fR } \fIexpr\fR
Returns the result of an integer comparison if both 
arguments are integers,
otherwise returns the result of a lexical comparison.
.TP .5i
\fIexpr \fR{ \fB+\fR, \fB\- \fR} \fIexpr\fR
Addition or subtraction of integer-valued arguments.
.TP .5i
\fIexpr \fR{ \fB\(**\fR, \fB/\fR, \fB% \fR} \fIexpr\fR
Multiplication, division, or remainder of the integer-valued arguments.
.TP .5i
\fIexpr\fB : \fIexpr\fR
The matching operator \fB:\fP compares the first argument
with the second argument which must be a regular expression;
regular expression syntax is the same as that of
.IR ed (C),
except that all patterns are ``anchored'' (i.e., begin with a
caret (\f3^\fP))
and therefore the caret is not a special character in that context.
(Note that in the shell, the caret has the same meaning as the
pipe symbol (\||\|).)
Normally the matching operator 
returns the number of characters matched
(zero on failure).
Alternatively, the
\fB\e(\|.\|.\|.\|\e)\fP
pattern symbols can be used to return a portion of the
first argument.
.SH Examples
.TP
1.
a=\*`expr\| $a\| +\| 1\*`
.PP
.RS
.RS
Adds 1 to the shell variable
.BR a .
.RE
.RE
.TP
2.
\fB#\fP  \*'For $a equal to either "/usr/abc/file" or just "/file"\*'
.br
\'expr\  $a\  \fB:\fP\  \*'\fB.\fP\(**/\e(\fB.\fP\(**\e)\*'\  \^| \ $a\'
.PP
.RS
.RS
Returns the last segment of a pathname
(i.e., \fPfile\fP).
Watch out for the slash alone as an argument:
.I expr\^
will take it as the division operator
(see
.I Notes
below).
.RE
.RE
'\"	.ne 6
'\"	.TP
'\"	3.
'\"	\fB#\fP \ A better representation of example 2.
'\"	.br
'\"	\'expr\  //$a\  \fB:\fP\  \fB.\fP\(**/\e(\fB.\fP\(**\e)\'
'\"	.PP
'\"	.RS
'\"	.RS
'\"	The addition of the
'\"	.B //
'\"	characters eliminates any ambiguity about the 
'\"	division operator and simplifies
'\"	the whole expression.
'\"	.RE
'\"	.RE
.TP
3.
expr \ \s-1$VAR\s0 \ \fB:\fP \ \'\fB.\(**\fP\'
.PP
.RS
.RS
Returns the number of characters in
.SM
.BR $VAR .
.RE
.RE
.SH "See Also"
ed(C), sh(C)
.SH Diagnostics
As a side effect of expression evaluation,
.I expr\^
returns the following exit values:
.sp
	0	If the expression is neither null nor zero
.br
	1	If the expression is null or zero
.br
	2	For invalid expressions
.PP
Other diagnostics include:
.TP 1.5i
.I syntax error\^
For operator/operand errors
.TP
.I nonnumeric argument
If arithmetic is attempted on such a string
.bp
.SH Notes
After argument processing by the shell,
.I expr\^
cannot tell the difference between an operator and an operand
except by the value.
If
.B $a
is an equals sign (=),
the command:
.PP
	expr \ $a \ = \ \*'=\*'
.PP
looks like:
.PP
	expr \ = \ = \ =
.PP
Thus the arguments are passed to
.I expr\^
(and will all be taken as the
.B =
operator).
The following permits comparing equals signs:
.PP
	expr \ X$a \ = \ X=
