.\" bm# 0000002088
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)getopt.C	1.1);


.ds mc 
.ds rv Revision 1
.TH GETOPT C
.SH Name
getopt \- Parses command options.
.SH Syntax
.B set \-\^\- \`getopt optstring $\(**\`
.SH Description
.if t .ds ' \h@.05m@\s+4\v@.333m@\'\v@-.333m@\s-4\h@.05m@
.if n .ds ' '
.if t .ds ` \h@.05m@\s+4\v@.333m@\`\v@-.333m@\s-4\h@.05m@
.if n .ds ` `
.I Getopt\^
is used to check and break up options in command lines 
for parsing by shell procedures.
.I Optstring\^
is a string of recognized option letters (see 
.IR getopt\^ (S)).
If a letter is followed by a colon, the option
is expected to have an argument which may or
may not be separated from it by whitespace.
The special option \fB\-\^\-\fP 
is used to delimit the end of the options.
.I Getopt\^
will place \fB\-\^\-\fP in the arguments at the end
of the options, or recognize it if used explicitly.
The shell arguments ($1 $2 . . .) are reset so that each option
is preceded by a dash 
.RB ( \- ) 
and in its own shell argument; each option
argument is also in its own shell argument.
.SH Example
The following code fragment shows how one can process the arguments
for a command that can take the options
.B a
and
.BR b ,
and the option
.BR o ,
which requires an argument:
.PP
.RS
.nf
.ss 18
set \-\ \- \*`getopt abo: $\(**\*`
if [ $? != 0 ]
then
	echo $USAGE
	exit 2
fi
for i in $\(**
do
	case $i in
	\-a | \-b)	FLAG=$i; shift;;
	\-o)		OARG=$2;	shift; shift;;
	\-\ \-)		shift;	break;;
	esac
done
.fi
.ss 12
.RE
.PP
This code will accept any of the following as equivalent:
.PP
.RS
.nf
.ss 18
cmd \-aoarg file file
cmd \-a \-o arg file file
cmd \-oarg \-a file file
cmd \-a \-oarg \-\- file file
.fi
.ss 12
.RE
.SH "See Also"
sh(C), getopt(S)
.SH Diagnostics
.I Getopt\^
prints an error message on
the standard error
when it encounters an option letter not included in
.IR optstring .
