.\" bm# 0000002176
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)sort.C	1.1);


.ds mc 
.ds rv Revision 1
.TH SORT C
.SH Name
sort \- Sorts and merges files.
.SH Syntax
.B sort
[
.BR \-cmubdf\&inrt x
] [
.BR + pos1
[
.BR \- pos2
] ] .\|.\|. [
.B \-o
output\ ] [\ files\ ]
.SH Description
.I Sort\^
merges and sorts lines from all named files
and writes the result on the standard output.
A dash
.RB ( \- )
may appear as a file in the
.I files\^
argument signifying the standard input.
If no input files are named, the standard input is sorted.
.PP
The default sort key is an entire line.
Default ordering is
lexicographic by bytes in machine
collating sequence.
The ordering is affected globally by the following options,
one or more of which may appear.
.TP 5
.B \-b
Ignores leading blanks (spaces and tabs) in field comparisons.
.TP
.B \-d
``Dictionary'' order: only letters, digits and blanks
are significant in comparisons.
.TP
.B \-f
Folds uppercase
letters onto lowercase.
.TP
.B \-i
Ignores characters outside the
.SM ASCII
octal range 040-0176 in nonnumeric comparisons.
.TP
.B \-n
An initial numeric string,
consisting of optional blanks, optional minus sign,
and zero or more digits with optional decimal point,
is sorted by arithmetic value.
Option
.B n
implies option
.BR b .
.TP
.B \-r
Reverses the sense of comparisons.
.TP
.BI \-t x\^
``Tab character'' separating fields is
.IR x .
.PP
The notation
.BI + "pos1\| " \- pos2\^
restricts a sort key to a field beginning at
.I pos1\^
and ending just before
.IR pos2 .
.I Pos1\^
and
.I pos2\^
each have the form
.IB m . n\^\f1,
optionally followed by one or more of the flags
.BR bdf\&inr ,
where
.I m\^
tells a number of fields to skip from the beginning of the line and
.I n\^
tells a number of characters to skip further.
If any flags are present they override all the global
ordering options for this key.
If the
.B b
option is in effect
.I n\^
is counted from the first nonblank in the field;
.B b
is attached independently to
.IR pos2 .
A missing
.BI \&. n\^
means
.BR \&. 0;
a missing
.BI \- pos2\^
means the end of the line.
Under the
.BI \-t x\^
option, fields are strings separated by
.IR x ;
otherwise fields are
nonempty nonblank strings separated by blanks.
.ne 1i
.PP
When there are multiple sort keys, later keys
are compared only after all earlier keys
compare equal.
Lines that otherwise compare equal are ordered
with all bytes significant.
Very long lines are silently truncated.
.PP
These option arguments are also understood:
.TP 5
.B \-c
Checks that the input file is sorted according to the ordering rules;
gives no output unless the file is out of sort.
.TP
.B \-m
Merges only, the input files are already sorted.
.TP
.B \-u
Suppresses all but one in each
set of duplicated lines.
Ignored bytes
and bytes outside keys
do not participate in
this comparison.
.TP
.B \-o
The next argument is the name of an output file
to use instead of the standard output.
This file may be the same as one of the inputs.
.SH Examples
The following prints in alphabetical order all the unique spellings
in a list of words
(capitalized words differ from uncapitalized):
.IP
sort \|\-u \|+0f \|+0 \|list
.PP
The following prints the password file
.RI ( passwd (M))
sorted by user
.SM ID
(the third colon-separated field):
.IP
sort \|\-t: \|+2n \|/etc/passwd
.PP
The following prints the first instance of each month 
in an already sorted file
of (month-day) entries
(the options
.B \-um
with just one input file make the choice of a
unique representative from a set of equal lines predictable):
.IP
sort \|\-um \|+0 \|\-1 \|dates
.SH Files
/usr/tmp/stm???
.bp
.SH "See Also"
comm(C),
join(C),
uniq(C)
.ne 1i
.SH Diagnostics
Comments and exits with nonzero status for various trouble
conditions and for disorder discovered under option
.BR \-c .
'\"	.SH Notes
