.\" bm# 0000002168
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)tar.C	1.1);


.ds mc 
.ds rv Revision 1
.TH TAR C
.SH Name
tar \- Archives files.
.SH Syntax
.B tar
[ key ] [ files ]
.SH Description
.I Tar\^
saves and restores files to and from an archive medium,
which is typically a storage device such as floppy
disk or tape, or a regular file.
Its actions are controlled by the
.I key\^
argument.
The
.I key\^
is a string of characters containing
at most one function letter and possibly
one or more function modifiers.
Valid function letters are
.BR c ,
.BR t ,
.BR x,
.BR u ,
and
.BR r .
Other arguments to the command are
.I files\^
(or directory
names)
specifying which files are to be backed up or restored.
In all cases, appearance of a directory name refers to
the files and (recursively) subdirectories of that directory.
.PP
The function portion of
the key is specified by one of the following letters:
.TP 8
.B r
The named
.I files\^
are written to the end of the archive.
The
.B c
function implies this function.
.TP
.B x
The named
.I files\^
are extracted from the archive.
If a named file matches a directory whose contents
had been written onto the archive, 
this directory is (recursively) extracted.
The owner, modification time, and mode are restored (if possible).
If no
.I files\^
argument is given, the entire contents of the
archive are extracted.
Note that if several files with the same name
are on the archive, the last one overwrites
all earlier ones.
.TP
.B t
The names of the specified files are listed each time that they occur
on the archive.
If no
.I files\^
argument is given,
all the names
on the archive are listed.
.TP
.B u
The named
.I files\^
are added to the archive if they
are not already there, or if they have
been modified since last written on that archive.
.TP
.B c
Creates a new archive; writing begins at the beginning
of the archive, instead of after the last file.
This command implies
the
.B r
function.
.PD
.PP
The following characters may be used in addition to the letter
that selects the desired function:
.TP 8
.B 0,.\^.\^.\^,7
This modifier selects the drive on which the archive
is mounted.  The default is
.BR 1 .
.TP
.B v
Normally,
.I tar\^
does its work silently.
The
.B v
(verbose)
option causes it to type the name of each file it treats,
preceded by the function letter.
With the
.B t
function,
.B v
gives more information about the
archive entries than just the name.
.TP
.B w
Causes
.I tar\^
to print the action to be taken, 
followed by the name of the file, and then
wait for the user's confirmation.
If a word beginning with
.B y
is given, the action is performed.
Any other input means
``no''.
.TP
.B f
Causes
.I tar\^
to use the next argument as the name of the archive instead
of
.BR /dev/mt? .
If the name of the file is a dash
.RB ( \- ),
.I tar\^
writes to the standard output or reads 
from the standard input, whichever is appropriate.
Thus,
.I tar\^
can be used as the head or tail of a pipeline.
.I Tar\^
can also be used to move hierarchies with the command:
.RS
.sp 10p
cd \|fromdir; \|tar \|cf \|\- \|\f3.\fP | \
\|(cd \|todir; \|tar \|xf \|\-)
.RE
.TP 8
.B b
Causes
.I tar\^
to use the next argument as the blocking factor for archive records.
The default is 1, the maximum is 20.
This option should only be used with 
raw magnetic tape archives (see
.B f
above).
The block size is determined automatically when reading
tapes (key letters
.B x
and
.BR t ).
.TP
.B F
Causes
.I tar\^
to use the next argument as the name of a file from
which succeeding arguments are taken.
A lone dash (\-) signifies that arguments will be
taken from the standard input.
.TP
.B l
Tells
.I tar\^
to print an error message if it cannot resolve all of the links
to the files being backed up.
If
.B l
is not specified, no
error messages are printed.
.TP
.B m
Tells
.I tar\^
to not restore the modification times.
The modification time
of the file
will be the time of extraction.
.TP 8
.B k
Causes
.I tar
to use the next argument as the size of an archive volume in kilobytes.
The minimum value allowed is 250.
This option
is useful when the archive is not intended for a magnetic tape
device, but for some fixed size device,
such as floppy disk (See
.B f
above).  
Very large files are split into ``extents'' across volumes.
When restoring from a multivolume archive,
.I tar
only prompts for a new volume 
if a split file has been partially restored.
.TP 8
.B n
Indicates the archive device is not a magnetic tape.  
The
.B k
option implies this.  
Listing and extracting the contents of
an archive are sped because 
.I tar\^ 
can seek over files it wishes to skip.
Sizes are printed in kilobytes instead of tape blocks.
.TP 8
.B p
Indicates that files are extracted using their
original permissions.
It is possible that a non-super-user may be unable
to extract files because of the permissions
associated with the files or directories being extracted.
.SH Examples
If the name of a floppy disk device is 
.BR /dev/fd1 ,
then a tar format file 
can be created on this device by typing:
.PP
.RS
.nf
assign /dev/fd
tar cvfk /dev/fd1 360 files
.fi
.RE
.PP
where 
.I files
are the names of files you want archived
and 360 is the capacity of the floppy disk in
kilobytes.
Note that arguments to key letters are given in the
same order as the key letters themselves, thus the
.B fk
key letters
have corresponding arguments
.B /dev/fd1
and 
.BR 360 .
Note that if a 
.I file
is a directory then the contents of the directory
are recursively archived.
To print a listing of the archive, type:
.PP
.RS
.nf
tar tvf /dev/fd1
.fi
.RE
.PP
At some later time you will likely want to extract
the files from the archive floppy.
You can do this by typing:
.PP
.RS
.nf
tar xvf /dev/fd1
.fi
.RE
.PP
The above command extracts all files from the
archive using the exact same pathnames as used when
the archive was created. Because of this behavior,
it is normally best to save archive files with relative
pathnames rather than absolute ones, since
directory permissions may not let you read the files
into the absolute directories specified.
.PP
In the above examples, the 
.B v
verbose option is used simply to confirm the reading
or writing of archive files on the screen. 
Also, a normal file could be substituted for the floppy device 
.BR /dev/fd1 
in the examples.
.PD
.SH Files
/etc/default/dump	Default devices
.PP
/tmp/tar\(**
.bp
.SH Diagnostics
Prints an error message about bad key characters and archive read/write errors.
.sp 10p
Prints an error message if not enough memory is available to hold the link tables.
.SH Notes
There is no way to ask for the
.IR n th
occurrence of a file.
.PP
The
.B u
option can be slow.
.PP
The
.B b
option should not be used with archives that are
going to be updated.
'\"	The current magnetic tape driver cannot
'\"	backspace raw magnetic tape.
If the archive is on a disk file, the
.B b
option should not be used at all, because updating
an archive stored on disk can destroy it.
In order to update (\fBr\fR or
.B u
option) a tar archive, do not use raw magtape and 
do not use the
.B b
option.  
This applies both when updating and when the archive 
was first created.
.PP
The limit on filename length is 100 characters.
.PP
Systems with a 1K-byte file system 
cannot specify raw disk devices unless the
.B b
option is used to specify an even number of blocks.  
This means that one cannot update a raw-mode disk partition.
used.  
.PP
Don't do:
.IP
tar xfF \- \-
.PP
This would imply taking two things from the standard input
at the same time.

