.\" bm# 0000002140
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)vi.C	1.1);


.ds mc 
.ds rv Revision 1
.hw non-white-space 
.TH VI C
.SH Name
vi \- Invokes a screen-oriented display editor.
.SH Syntax
\fBvi\fR [ \-\fIoption\fR... ] [ \fIcommand\fR ] [ \fIfilename\fR]
.SH Description
Vi offers a powerful set of text editing
operations based on a set of mnemonic commands.
Most commands are single keystrokes that perform
simple editing functions.
Vi displays a full screen ``window''
into the file you are editing.
The contents of this window can 
be changed quickly and easily within vi.
While editing, visual feedback is provided 
(the name vi itself is short for 
``visual'' ).
.PP
Vi and the line editor ex are one and the same editor:
the names vi and ex identify a particular user interface
rather than any underlying functional difference.
The differences in user interface, however, are
quite striking.  
Ex is a powerful line-oriented editor,  
similar to the editor ed.
However, in both ex and ed, visual
updating of the terminal screen is limited,
and commands are entered on a command line.
Vi, on the other hand, is a screen-oriented editor
designed so that what you see on the screen corresponds
exactly and immediately to the contents 
of the file you are editing.
.PP
Options available on the vi command line:
.TP 10n
\-t
Equivalent to an initial \fItag\fR command;
edits the file containing the tag
and positions the editor at its definition.
.TP 
\-r
Used in recovering after an editor or system crash,
retrieving the last saved version of the named file. 
If no file is specified, this option prints a list of saved files.
.TP
\-l
Specific to editing LISP, this option sets the showmatch
and lisp options.
.TP 
\-w\fIn\fR
Sets the default window size to
.IR n .
Useful on dialups to start in small windows.
.TP 
\-x
Causes vi to prompt for a key 
used to encrypt and decrypt the contents of the named files.
.TP 
\-R
Sets a readonly option so that files can be
viewed but not edited.
.SS \fIThe Editing Buffer\fR
.PP
Vi performs no editing operations on the file
that you name during invocation.
Instead, it works on a copy of the file in an
.I "editing buffer."
The editor remembers the name of the file
specified at invocation,
so that it can later copy the editing
buffer back to the named file.
The contents of the named file are not affected
until the changes are copied
back to the original file.
This allows editing of the buffer without
immediately destroying the contents of the original file.
.PP
When you invoke vi with a single filename argument,
the named file is copied to a temporary editing buffer.
When the file is written out, the temporary file is
written back to the named file.
.SS \fIModes of Operation\fR
Within vi there are three distinct modes of
operation:
.TP 24n
Command Mode
Within command mode, signals from the keyboard are
interpreted as editing commands.
.TP 
Insert Mode
Insert mode can be entered by typing any of the vi
insert, append, open, substitute, change, or replace commands.
Once in insert mode, letters typed at the keyboard are
inserted into the editing buffer.
.TP 24n 
Ex Escape Mode
The vi and ex editors are one and the same editor
differing mainly in their user interface.
In vi commands are usually single
keystrokes.
In ex, commands are lines of text terminated by a 
\s-1RETURN\s0.
Vi has a special ``escape''
command that gives access to many of these
line-oriented ex commands.
To escape to ex escape mode,
type a colon (:).
The colon is echoed on the status line as a
prompt for the ex command.
An executing command can be
aborted by pressing \s-1INTERRUPT\s0.
Most file manipulation commands are executed in
ex escape mode; for example, the commands to
read in a file, and to write out the editing buffer to a file.
.SS "\fISpecial Keys\fR"
There are several special keys in vi.  
These keys are used to
edit, delimit, or abort commands and command lines.
.TP 8
\s-1ESC\s0 
Used to return to vi command mode,
cancel partially formed commands.
.TP 
\s-1RETURN\s0 
Terminates ex commands
when in ex escape mode.
Also used to start a new
line when in insert mode.
.TP 
\s-1INTERRUPT\s0
Often the same as the \s-1DEL\s0 or \s-1RUBOUT\s0
key on many terminals. 
Generates an interrupt, telling the editor to stop what it is doing.
Used to 
abort any command that is executing.
.TP 
\^/\^
Used to specify 
a string to be searched for.  
The slash appears
on the status line as a prompt for a search string.
The question mark (?) works exactly like
the slash key, except that it is used to search
backward in a file instead of forward.
.TP 
:
The colon is a prompt for an ex command.
You can then type in any ex command, followed by
an \s-1ESC\s0 or \s-1RETURN\s0
and the given ex command is executed.
.P
The following characters are special in insert mode:
.TP 10n
\s-1BKSP\s0
Backs up the cursor one character on the current line.
The last character typed before the \s-1BKSP\s0
is removed from the input buffer, but remains
displayed on the screen.
.TP
\s-1CNTRL-U\s0
Moves the cursor back to the first character of the insertion,
and restarts insertion.
.TP
\s-1CNTRL-V\s0
Removes the special significance of the next typed
character.
Use \s-1CNTRL-V\s0
to insert control characters.
Line feed and  \s-1CNTRL-J\s0
cannot be inserted
in the text except as newline characters.
\s-1CNTRL-Q\s0
and \s-1CNTRL-S\s0
are trapped by the operating system
before they are interpreted by vi, so they too cannot
be inserted as text.
.TP
\s-1CNTRL-W\s0
Moves the cursor back to the first character of the
last inserted word.
.TP
\s-1CNTRL-T\s0
During an insertion, with the 
.I autoindent
option
set and at the beginning of the current
line, typing this character will insert 
.I shiftwidth 
whitespace.
.TP 
\s-1CNTRL-@\s0
If typed as the first character of an insertion 
it is replaced with the
last text inserted, and the insertion terminates.
Only 128 characters are
saved from the last insertion.
If more than 128 characters were inserted, then
this command inserts no characters.
A \s-1CNTRL-@\s0 
cannot be part of a file, even if quoted.
.SS \fIInvoking and Exiting Vi\fR
To enter vi type:
.TP 20
vi
\fIEdits empty editing buffer\fP
.TP 
vi file
\fIEdits named file\fP
.TP 
vi +123 file
\fIGoes to line 123\fR
.TP 
vi +45 file
\fIGoes to line 45\fR
.TP 
vi +/word file
\fIFinds first occurrence of ``word''\fR
.TP 
vi +/tty file
\fIFinds first occurrence of ``tty''\fR
.PP
There are several ways to exit the editor:
.TP 5
.TP :wq
This command writes the editing buffer to the file
you are editing, quits the editor, and returns
to the \*(x1 shell.
.TP 
ZZ
The editing
buffer is written to the file 
.I only 
if any changes were made.
.TP 
:x
The editing buffer is written to the file
.I only
if any changes were made.
.TP 
:q!
Cancels an editing session.
The exclamation mark (!) tells vi to quit unconditionally.
In this case, the editing buffer is not written out.
.SH "Vi Commands"
Vi is a visual editor with a window on the file.  
What you see on the screen is vi's notion of
what the file contains.
Commands do not cause any change to the screen until the
complete command is typed.  
Most commands may take a preceding count that
specifies repetition of the command.
This count parameter is not given in
the following command descriptions, 
but is implied unless overriden by some
other prefix argument.
When vi gets an improperly formatted command
it rings a bell.
.SS \fICursor Movement\fR
The cursor movement keys allow you to move your
cursor around in a file.
Note in particular the arrow
keys (if available on your terminal), 
the ``h'' ``j'', ``k'' , and ``l''
cursor keys, and \s-1SPACE, BKSP, CNTRL-N\s0, and \s-1CNTRL-P\s0.
These three sets of keys perform identical
functions.
.P
.B "Forward Space \- l, SPACE, or -->"
.TP 10n
Syntax:
.B l
.br
\s-1\fBSPACE\fR\s0
.br
.B -->
.TP 10n
Function:
Moves the cursor forward one character.  
If a count is given, move forward 
count characters.
You cannot move past the end of the line.
.P
.B "Backspace \- h, \s-1BKSP\s0, or <--"
.TP 10n 
Syntax:
.B h
.br
\s-1\fBBKSP\fR\s0
.br
.B <--
.TP 10n 
Function:
Moves cursor backward one character.
If a count is given, moves backward
\fIcount\fR characters.
Note that you cannot move past the beginning of
the current line.
.P
.B "Next Line \- +, \s-1RETURN\s0, j, \
\s-1CNTRL-N\s0, and \s-1LF\s0"
.TP 10n
Syntax:
.B +
.br
\s-1\fBRETURN\fR\s0
.TP 10n 
Function:
Moves the cursor down to the beginning of the next line.
.TP 10n 
Syntax:
.B j
.br
\s-1\fBCNTRL-N\fR\s0
.br
\s-1\fBLF\fR\s0
.br
.B "(down arrow)"
.TP 10n 
Function:
Moves the cursor down one line, remaining in the same column.
Note the difference between these commands and the
preceding set of next line commands which move to
the 
.I beginning 
of the next line.
.P
.B "Previous Line \- k, \s-1CNTRL-P\s0, and \-"
.TP 10n 
Syntax:
.B k
.br
\s-1\fBCNTRL-P\fR\s0
.br
.B "(up arrow)"
.br
.TP 10n 
Function:
Moves the cursor up one line, remaining in the same column.
If a count is given then the cursor is moved \fIcount\fR
lines.
.TP 10n 
Syntax:
.B \- 
.TP 10n 
Function:
Moves the cursor up to the beginning of the previous line.
If a count is given then the cursor is moved up a
\fIcount\fR lines.
.P
.B "Beginning of Line \- 0 and ^"
.TP 10n 
Syntax:
.B ^
.br
.B 0
.TP 10n 
Function:
Moves the cursor to the beginning of the current line.
Note that 
.B 0 
always moves the cursor to the first
character of the current line. 
The caret 
.RB (\| ^ \|)
works somewhat differently: it
moves to the first character on a line that is not a tab or a space.
This is useful when editing files that have a great
deal of indentation, such as program texts.
.P
.B "End of Line \- $"
.TP 10n 
Syntax:
.B $
.TP 10n 
Function:
Moves the cursor to the end of the current line.
Note that the cursor resides on top of the last
character on the line.
If a count is given, 
then the cursor is moved forward \fIcount\fR\-1 
lines to the end of the line.
.P
.B "Goto Line \- G"
.TP 10n 
Syntax:
.B [\fIlinenumber\fP]G
.TP 10n
Function:
Moves the cursor to the beginning of the line specified by
.IR linenumber .
If no 
.I linenumber
is given, the cursor moves to the beginning of the
.I last 
line in the file.
To find the line number of the current line, use 
\s-1CNTRL-G\s0.
.P
.B "Column \- |"
.TP 10n 
Syntax:
.B [\fIcolumn\fP]|
.TP 10n
Function:
Moves the cursor to the 
column in the current line given by 
.IR column .
If no 
.I column
is given then the cursor is moved to the
first column in the current line.
.P
.B "Word Forward \- w and W"
.TP 10n 
Syntax:
.B w
.br
.B W
.TP 10n 
Function:
Moves the cursor forward to the beginning of the next word.
The lowercase
.B w 
command
searches for a word defined as a 
string of alphanumeric characters separated by
punctuation or whitespace (i.e., tab, newline, or space characters).
The uppercase
.B W 
command searches for a word defined as a 
string of nonwhitespace characters.
.P
.B "Back Word \- b and B"
.TP 10n 
Syntax:
.B b
.br
.B B
.TP 10n 
Function:
Moves the cursor backward to the beginning of a word.
The lowercase
.B b 
command searches backward for a word defined as a 
string of alphanumeric characters separated by
punctuation or whitespace (i.e., tab, newline, or space characters).
The uppercase
.B B 
command searches for a word defined as a 
string of non-whitespace characters.
If the cursor is already within a word,
then it moves backward to the beginning of that word.
.P
.B "End \- e and E"
.TP 10n 
Syntax:
.B e
.br
.B E
.TP 10n 
Function:
Moves the cursor to the end of a word.
The lowercase
.B e 
command moves the cursor to the last character of
a word, where a word is defined as a 
string of alphanumeric characters separated by
punctuation or whitespace (i.e., tab, newline, or space characters).
The uppercase
.B E 
moves the cursor to the last character of a word
where a word is defined as a 
string of nonwhitespace characters.
If the cursor is already within a word,
then it moves to the end of that word.
.P
.B "Sentence \- ( and )"
.TP 10n 
Syntax:
.B (
.br
.B )
.TP 10n 
Function:
Moves the cursor to the beginning (left parenthesis)
or end 
of a sentence (right parenthesis).
A sentence is defined as a sequence of characters
ending with a period (\|.\|),
question mark (?), or exclamation mark (!), followed by
either two spaces or a newline.
A sentence begins on the
first nonwhitespace character following a preceding sentence.
Sentences are also delimited by paragraph and
section delimiters. See below.
.P
.B "Paragraph \- { and }"
.TP 10n 
Syntax:
.B }
.br
.B {
.TP 10n 
Function:
Moves the cursor to the beginning 
.RB ( { ) 
or end 
.RB ( } )
of a paragraph.
A paragraph is  defined with the 
.I paragraphs 
option.
By default, paragraphs are delimited by the nroff macros 
``.IP'', ``.LP'', ``.P'', ``.QP'',
and ``.bp''.
Paragraphs also begin after empty lines.
.P
.B "Section \- [[ and ]]"
.TP 10n 
Syntax:
.B ]]
.br
.B [[
.TP 10n 
Function:
Moves the cursor to the beginning 
.RB ( [[ ) 
or end 
.RB ( ]] )
of a section.
A section is  defined with the 
.I sections 
option.
By default, sections are delimited by the nroff macros 
``.NH''
and 
``.SH''.
Sections also start at formfeeds (\s-1CNTRL-L\s0)
and at lines beginning with a brace ({).
.P
.B "Match Delimiter \- %"
.TP 10n 
Syntax:
.B %
.TP 10n 
Function:
Moves the  cursor to a matching delimiter, where
a delimiter is a parenthesis, a bracket, or a brace.
This is useful when matching pairs 
of nested parentheses, brackets, and braces.
.P
.B "Home \- H"
.TP 10n 
Syntax:
.B [\fIoffset\fB]H
.TP 10n 
Function:
Moves the  cursor to upper left corner of screen.
Use this command to quickly move to the top of the screen.
If an 
.I offset 
is given, then the cursor is homed 
.IR offset -1 
number of lines from the top of the screen.
Note that the command 
``dH''
deletes all lines from
the current line to the top line shown on the screen.
.P
.B "Middle Screen \- M"
.TP 10n 
Syntax:
.B M
.TP 10n 
Function:
Moves the cursor to the beginning of the screen's middle line. 
Use this command to quickly move to the middle of
the screen from either the top or the bottom.
Note that the command 
``dM''
deletes from the
current line to the line specified by the 
.B M 
command.
.P
.B "Lower Screen \- L"
.TP 10n 
Syntax:
.B [\fIoffset\fR]L
.TP 10n 
Function:
Moves the  cursor to the lowest line on the screen.
Use this command to quickly move to the bottom of the screen.
If an 
.I offset 
is given, then the cursor is homed 
.IR offset -1
number of lines from the bottom of the screen.
Note that the command 
``dL''
deletes all lines from
the current line to the bottom line shown on the screen.
.P
.B "Previous Context \- \`\` and \'\' "
.TP 10n 
Syntax:
.B \'\'
.br
.B \'\fIcharacter\fR
.br
.B \`\`
.br
.B \`\fIcharacter\fR
.TP 10n 
Function:
Moves the cursor to previous context or to context
marked with the 
.B m
command.
If the single quotation mark or back quotation mark is doubled, then
the cursor is moved to previous context.
If a single character is given after either quotation mark, 
then the cursor is moved to the location of the specified mark
as defined by the
.B m
command.
Previous context is the location in the file of the
last ``nonrelative'' cursor movement.
The single quotation mark (\|\'\|) syntax
is used to move to the beginning of the
line representing the previous context. 
The back quotation mark (\|\`\|) syntax
is used to move to the previous context
.I within 
a line.
.SS "\fIThe Screen Commands\fR"
The screen commands are
.I not 
cursor movement commands and cannot be used in
delete commands as the delimiters of text objects.
However, the screen commands do move the cursor and
are useful in paging or scrolling through a file.
These commands are described below:
.P
.B "Page \- \s-1CNTRL-U\s0 and \s-1CNTRL-D\s0"
.TP 10n
Syntax:
\s-1\fB[\fIsize\fB]CNTRL-U\fR\s0
.br
\s-1\fB[\fIsize\fB]CNTRL-D\fR\s0
.TP 10n 
Function:
Scrolls the screen up a half window
(\s-1CNTRL-U\s0) or down a half window (\s-1CNTRL-D\s0).
If 
.I size 
is given, then the scroll is 
.I size 
number of lines.
This value is remembered for all later scrolling
commands.
.P
.B "Scroll \- \s-1CNTRL-F\s0 and \s-1CNTRL-B\s0"
.TP 10n 
Syntax:
\s-1\fBCNTRL-F\fR\s0
.br
\s-1\fBCNTRL-B\fR\s0
.TP 10n 
Function:
Pages screen forward and backward.
Two lines of continuity are kept between pages if
possible.
A preceding count gives the number of pages to move
forward or backward.
.P
.B "Status \- \s-1CNTRL-G\s0"
.TP 10n 
Syntax:
\s-1\fBBELL\fR\s0
.br
\s-1\fBCNTRL-G\fR
.TP 10n 
Function:
Prints vi status on status line.
This gives you the name of the file you are editing,
whether it has been modified,
the current line number, the number of lines in the file, 
and the percentage of the file (in lines) that precedes the
cursor.
.P
.B "Zero Screen \- z"
.TP 10n 
Syntax:
.B [\fIlinenumber\fP]z[\fIsize\fP\]\s-1RETURN\s0
.br
.B [\fIlinenumber\fP]z[\fIsize\fP\]\s+2.\s0
.br
.B [\fIlinenumber\fP]z[\fIsize\fP\]\-
.br
.TP 10n 
Function:
Redraws the display with the current line placed at
or ``zeroed''
at the top, middle, or bottom of the screen,
respectively.
If you give a
.IR size ,
then the number of lines displayed is equal to
.IR size .
If a preceding 
.I linenumber 
is given, then the given line is
placed at the top of the screen.
If the last argument is a \s-1RETURN\s0,
then the current line is placed at the top of the screen.
If the last argument is a period (\|.\|),
then the current line is placed in the middle of the screen.
If the last argument is a minus sign (\-),
then the current line is placed at the bottom of the screen.
.P
.B "Redraw \- \s-1CNTRL-R\s0 or \s-1CNTRL-L\s0" 
.TP 10n 
Syntax:
\s-1\fBCNTRL-R\fR\s0
.br
\s-1\fBCNTRL-L\fR\s0
.TP 10n 
Function:
Redraws the screen.
Use this command to erase any system messages that
may scramble your screen. 
Note that system messages do not affect the file you are editing.
.sp
.SS "\fIText Insertion\fR"
The text insertion commands always place you in
insert mode.
Exit from insert mode is always done by pressing
\s-1ESC\s0.
The following insertion commands are 
``pure''
insertion commands; no text is deleted when you use them.
This differs from the text modification 
commands change, replace, and substitute, which 
delete and then insert text in one operation.
.P
.B "Insert \- i and I"
.TP 10n 
Syntax:
.B i[\fItext\fP]\s-1ESC\s0
.br
.B I[\fItext\fP]\s-1ESC\s0
.TP 10n 
Function:
Insert 
.I text
in editing buffer.
The lowercase
.B i
command places you in insert mode.
.I Text 
is inserted 
.I before
the character beneath the cursor.
To insert a newline, just press a
\s-1RETURN\s0.
Exit insert mode by typing the
\s-1ESC\s0 key.
The uppercase 
.B I
command places you in insert mode, but begins text insertion
at the beginning of the current line,
rather than before the cursor.
.P
.B "Append \- a and A"
.TP 10n 
Syntax:
.B a[\fItext\fP]\s-1ESC\s0
.br
.B A[\fItext\fP]\s-1ESC\s0
.TP 10n 
Function:
Appends
.I text
to the editing buffer.
The lowercase
.B a
command works 
.I exactly 
like the lowercase
.B i
command, except that
text insertion begins after the cursor and not before.
This is the one way to add text to the end of a line.
The uppercase
.B A
command begins appending text at the end of the
current line rather than after the cursor.
.P
.B "Open New Line \- o and O"
.TP 10n 
Syntax:
.B o[\fItext\fP]\s-1ESC\s0
.br
.B O[\fItext\fP]\s-1ESC\s0
.TP 10n 
Function:
Opens a new line and inserts text.
The lowercase
.B o
command opens a new line below the current line;
uppercase
.B O
opens a new line 
.I above
the current line.
After the new line has been opened,
both these commands work like the 
.B I
command.
.sp
.SS "\fIText Deletion\fR"
Many of the text deletion commands use 
the letter ``d'' as an operator.
This operator deletes text objects delimited by
the cursor and a cursor movement command.
Deleted text is always saved away in a buffer.
The delete commands are described below:
.P
.B "Delete Character \- x and X"
.TP 10n 
Syntax:
.B x
.br
.B X
.TP 10n 
Function:
Deletes a character.
The lowercase 
.B x
command deletes the character beneath the cursor.
With a preceding count,
.I count 
characters are deleted to the right beginning with
the character beneath the cursor.
This is a quick and easy way to delete a few characters.
The uppercase 
.B X
command deletes the character just before the cursor.
With a preceding count,
.I count 
characters are deleted backward, beginning with
the character just before the cursor.
.P
.B "Delete \- d and D"
.TP 10n 
Syntax:
.B d\fIcursor-movement\fR
.br
.B dd
.br
.B D
.TP 10n 
Function:
Deletes a text object.
The lowercase
.B d
command takes a 
.I cursor-movement 
as an argument.
If the 
.I cursor-movement 
is an intraline command, then
deletion takes place from the cursor to the end of
the text object delimited by the 
.I cursor-movement .
Deletion forward deletes the character beneath the 
cursor; deletion backward does not.
If the 
.I cursor-movement 
is a multiline command, then
deletion takes place from and including the current
line to the text object delimited by the 
.I cursor-movement .
.PP 
The 
.B dd
command deletes whole lines.
The uppercase
.B D
command deletes from and including the cursor 
to the end of the current line.
.PP
Deleted text is automatically pushed on a stack of
buffers numbered 1 through 9.
The most recently deleted text is also placed in a
special delete buffer that is logically buffer 0.
This special buffer is the default buffer for all
(put) commands using the double quotation mark
(") to specify the number of the buffer for
delete, put, and yank commands.
The buffers 1 through 9 can be accessed with the 
.B p
and
.B P
(put) commands by appending the double quotation mark
(") to the number of the buffer.
For example 
.IP
"4p
.PP
puts the contents of delete buffer number 4 in your
editing buffer just below the current line.
Note that the last deleted text is 
``put''
by default and does not need a preceding buffer number.
.sp
.SS "\fIText Modification\fR"
The text modification commands all involve the
replacement of text with other text.
This means that some text will necessarily be
deleted.
All text modification commands can be 
``undone'' with the 
.B u 
command, discussed below:
.P
.B "Undo \- u and U"
.TP 10n 
Syntax:
.B u 
.br
.B U 
.TP 10n 
Function:
Undoes the last insert or delete command.
The lowercase
.B u
command undoes the last insert or delete command.
This means that after an insert,
.B u
deletes text; and after a delete,
.B u
inserts text.
For the purposes of undo, all text modification
commands are considered insertions.
.sp
The uppercase
.B U
command restores the current line to its state
before it was edited, no matter how many times the
current line has been edited since you moved to it.
.P
.B "Repeat \- \fB.\fR"
.TP 10n 
Syntax:
.B \s+4\&.\s0 
.TP 10n 
Function:
Repeats the last insert or delete command.
A special case exists for repeating the 
.B p
and 
.B P
``put''
commands.
When these commands are preceded by the name of a
delete buffer, then successive
.B u 
commands print out the contents of the delete buffers.
.P
.B "Change \- c and C"
.TP 10n 
Syntax:
.B "c\fIcursor-movement\ text\fB\s-1ESC\s0"
.br
.B C\fItext\fB\s-1ESC\s0
.br
.B cc\fItext\fB\s-1ESC\s0
.TP 10n 
Function:
Changes a text object and replaces it with
.I text .
Text is inserted as with the 
.B i
command.
A dollar sign ($) marks the extent of the change.
The 
.B c
command changes arbitrary text objects delimited by
the cursor and a 
.I cursor-movement .
The 
.B C
and 
.B cc 
commands affect whole lines and are identical in
function.
.P
.B "Replace \- r and R"
.TP 10n 
Syntax:
.B r\fIchar\fP
.br
.B R\fItext\fP\s-1ESC\s0
.TP 10n 
Function:
Overstrikes character or line
with 
.I char
or
.I text ,
respectively.
Use
.B r
to overstrike a single character and
.B R
to overstrike a whole line.
A count multiplies the replacement text
count times.
.P
.B "Substitute \- s and S"
.TP 10n 
Syntax:
.B s\fItext\fP\s-1ESC\s0
.br
.B S\fItext\fP\s-1ESC\s0
.TP 10n 
Function:
Substitutes current character
or current line with 
.I text.
Use 
.B s
to replace a single character with new text.
Use 
.B S
to replace the current line with new text.
If a preceding count is given, then 
.I text 
substitutes for count number of characters or lines
depending on whether the command is
.B s
or
.BR S ,
respectively.
.P
.B "Filter \- !"
.TP 10n 
Syntax:
.B "!\fIcursor-movement\ cmd\s-1RETURN\s0"
.TP 10n 
Function:
Filters the text object delimited by the cursor and 
.I cursor-movement
through the \*(x1 command,
.IR cmd .
For example, the following command sorts all
lines between the cursor and the bottom of the screen,
substituting the designated lines with the sorted lines:
.sp
	!Lsort
.sp
Arguments and shell metacharacters may be included
as part of 
.IR cmd ;
however, standard input and output are always
associated with the text object being filtered.
.P
.ne 1i
.B "Join Lines \- J"
.TP 10n 
Syntax:
.B J 
.TP 10n 
Function:
Joins the current line with the following line.
If a count is given, then count lines are joined.
.P
.B "Shift \- < and >"
.TP 10n 
Syntax:
.B >[\fIcursor-movement\fP]
.br
.B <[\fIcursor-movement\fP]
.br
.B >>
.br
.B <<
.TP 10n 
Function:
Shifts text left (>) or right (<).
Text is shifted by the value of the option
.IR shiftwidth , 
which is normally set to eight spaces.
Both the
.B >
and 
.B <
commands shift all lines in the text object 
delimited by the current line and 
.IR cursor-movement .
The
.B >>
and 
.B <<
commands affect whole lines.
All versions of the command can take a preceding
count that acts to multiply the number of objects affected.
.SS "\fIText Movement\fR"
The text movement commands move text in and out of the
named buffers a-z and out of the delete buffers 1-9.
These commands either 
``yank''
text out of the editing
buffer and into a named buffer or 
``put''
text into the editing buffer from a named buffer or a
delete buffer.
By default, text is put and yanked from 
the 
``unnamed buffer'', 
which is also where the most recently deleted text
is placed.
Thus it is quite reasonable to delete text, 
move your cursor to the location where you want the
deleted text placed, and then put the text back into
the editing buffer at this new location 
with the 
.B p
or
.B P
command.
.PP
The named buffers are  most useful for keeping track
of several chunks of text that you want to keep on
hand for later access, movement, or rearrangement.
These buffers are named with the letters 
``a'' through ``z''.
To refer to one of these buffers (or one of the
numbered delete buffers) in a command such
as put, yank, or delete, use a quotation mark.
For example, to yank a line into the buffer named 
.IR a ,
type:
.IP
"ayy
.P
To put this text back into the file, type:
.IP
"ap
.P
If you delete text into the buffer named
.I A
rather than 
.IR a ,
then text is appended to the buffer.
.PP
Note that the contents of the
named buffers are not destroyed when you switch
files.
Therefore, you can delete or yank text into a
buffer, switch files, and then do a put. 
Buffer contents are
.I destroyed
when you exit the editor,
so be careful.
.P
.B "Put \- p and P"
.TP 10n 
Syntax:
.B ["\fIalphanumeric\fP]p 
.br
.B ["\fIalphanumeric\fP]P 
.TP 10n 
Function:
Puts text from a buffer into the editing buffer.
If no buffer name is specified, then text is put
from the unnamed buffer.
The lowercase 
.B p
command puts text either below the current line
or after the cursor, depending on whether the buffer
contains a partial line or not.
The uppercase 
.B P
command puts text either above the current line
or before the cursor, again depending on whether the buffer
contains a partial line or not.
.P
.B "Yank \- y and Y"
.TP 10n 
Syntax:
.B ["\fIletter\fP]y\fIcursor-movement\fR
.br
.B ["\fIletter\fP]yy
.br
.B ["\fIletter\fP]Y
.TP 10n 
Function:
Copies text in the editing buffer to a named buffer.
If no buffer name is specified, then text is yanked
into the unnamed buffer.
If an uppercase 
.I letter
is used,
then text is appended to the buffer and does not
overwrite and destroy the previous contents.
When a 
.I cursor-movement
is given as an argument, the
delimited text object is yanked.
The
.B Y
and 
.B yy
commands
yank a single line, or, if a preceding count is
given, multiple lines can be yanked.
.sp
.SS "\fISearching\fR"
The search commands search either forward or
backward in the editing buffer for 
text that matches a given regular expression.
.P
.B "Search \- / and ?"
.TP 10n 
Syntax:
.B /[\fIpattern\fP]/[\fIoffset\fP]\s-1RETURN\s0
.br
.B /[\fIpattern\fP]\s-1RETURN\s0
.br
.B ?[\fIpattern\fP]?[\fIoffset\fP]\s-1RETURN\s0
.br
.B ?[\fIpattern\fP]\s-1RETURN\s0
.TP 10n 
Function:
Searches forward (/) or backward (?) for 
.IR pattern .
A string is actually a regular expression.
The trailing delimiter is not required.
If no 
.I pattern
is given, 
then last 
.I pattern 
searched for is used.
After the second delimiter, an 
.I offset
may be
given, specifying the beginning of a line relative to the line on
which 
.I pattern 
was found.
For example
.sp
	/word/-
.sp
finds the beginning of the line immediately preceding 
the line containing 
``word''
and 
.sp
	/word/+2
.sp
finds the beginning of the line two lines after
the line containing 
``word''.
See also the 
.I ignorecase
and
.I magic
options.
.P
.B "Next String \- n and N"
.TP 10n 
Syntax:
.B n 
.br
.B N 
.TP 10n 
Function:
Repeats the last search command.
The 
.B n
command repeats the search in the same direction
as the last search command.
The 
.B N
command repeats the search in the opposite
direction of the last search command.
.P
.B "Find Character \- f and F"
.TP 10n 
Syntax:
.B f\fIchar\fR
.br
.B F\fIchar\fR 
.br
.B ;
.br
.B ,
.TP 10n 
Function:
Finds character \fIchar\fR on the current line.
The lowercase 
.B f
searches forward on the line;
the uppercase
.B F
searches backward.
The semicolon
.RB (\| ; \|) 
repeats the last character search.
The comma
.RB (\| , \|) 
reverses the direction of the search.
.P
.B "To Character \- t and T"
.TP 10n 
Syntax:
.B t\fIchar\fR 
.br
.B T\fIchar\fR 
.br
.B ;
.br
.B ,
.TP 10n 
Function:
Moves the cursor up to but not on to
.IR char .
The semicolon
.RB (\| ; \|) 
repeats the last character search.
The comma
.RB (\| , \|) 
reverses the direction of the search.
.P
.ne 1i
.B "Mark \- m"
.TP 10n 
Syntax:
.B m\fIletter\fR
.TP 10n 
Function:
Marks a place in the file with a lowercase
.IR letter .
You can move to a mark using the 
``to mark''
commands described below.
It is often useful to create a mark, move the
cursor,
and then delete from the cursor to the mark 
``a'' with the following command: 
.sp
	d\'a
.P
.B "To Mark \- \' and \`"
.TP 10n 
Syntax:
\'\fIletter\fP
.br
\`\fIletter\fP
.TP 10n 
Function:
Move to 
.IR letter .
These commands let you move to the location of a mark.
Marks are denoted by single lowercase alphabetic characters.
Before you can move to a mark, it must first
be created with the 
.B m
command.
The back quotation mark (\|\'\|) moves you to the exact location of the
mark within a line;
the forward quotation mark (\|\'\|) moves you to the beginning of the
line containing the mark.
Note that these commands are also legal cursor movement commands.
.SS "\fIExit and Escape Commands\fR"
There are several commands that are used to
escape from vi command mode and to exit the editor.
These are described below:
.P
.B "Ex Escape \- :"
.TP 10n 
Syntax:
.B :
.TP 10n 
Function:
Enters ex escape mode to execute an ex command.
The colon appears on the status line as a prompt for
an ex command.
You then can enter an ex command line terminated by
either a \s-1RETURN\s0
or an \s-1ESC\s0
and the ex command will execute.
You are then prompted to type \s-1RETURN\s0
to return to vi command mode.
During the input of the ex command line or during
execution of the ex command you may press \s-1INTERRUPT\s0
to abort what you are doing and return to vi command mode.
.P
.B "Exit Editor \- ZZ"
.TP 10n 
Syntax:
.B ZZ
.TP 10n 
Function:
Exit vi and write out the file if any changes have been made.
This returns you to the shell from which you invoked vi.
.br
.ne 1i
.P
.B "Quit to Ex \- Q"
.TP 10n 
Syntax:
.B Q
.TP 10n 
Function:
Enters the ex editor.
When you do this, you will still be editing the same file.
You can return to vi by typing the 
.B vi
command from ex.
.SH "Ex Commands"
Typing the colon (:) escape command when in
command mode, produces a colon prompt
on the status line. 
This prompt is for a command available in the 
line-oriented editor, ex.
In general, ex commands let you write out or read in files, 
escape to the shell, or switch editing files.
.P
Many of these commands perform actions that affect the ``current''
file by default.
The current file is normally the file that you named
when you invoked vi, although the current file can
be changed with the 
``file''
command,
.BR f ,
or with the 
``next''
command,
.BR n .
In most respects, these commands are identical to
similar commands for the editor,
.BR ed .
All such ex commands are aborted by either a 
\s-1RETURN\s0
or an
\s-1ESC\s0.
We shall use a
\s-1RETURN\s0
in our examples.
Command entry is terminated by typing an 
\s-1INTERRUPT\s0.
.SS "\fICommand Structure\fR"
Most ex command names are English words,
and initial prefixes of the words are acceptable abbreviations.
In descriptions, only the abbreviation is
discussed, since this is the most frequently used
form of the command.
The ambiguity of abbreviations is resolved 
in favor of the more commonly used commands.
As an example, the command 
.B substitute
can be abbreviated 
.B s
while the shortest available abbreviation for the 
.B set
command is 
.B se.
.P
Most commands accept prefix addresses 
specifying the lines in the file
that they are to affect.
A number of commands also may take a trailing
.I count
specifying the number of lines to be involved in the command.
Counts are rounded down if necessary.
Thus, the command 
``10p''
will print the 
tenth line in the buffer while
``move 5''
will move the current line after line 5.
.P
Some commands take other information or parameters,
stated after the command name.
Examples might be option names in a
.B set
command, such as 
``set number'',
a filename in an
.B edit
command, a regular expression in a
.B substitute
command, or a target address for a
.B copy
command, such as
.IP
1,5 copy 25
.P
A number of commands have variants.
The variant form of the command is invoked by placing an
exclamation mark (!) immediately after the command name.
Some of the default variants may be controlled by options;
in this case, the exclamation mark turns off the
meaning of the default.
.P
In addition, many commands take flags, including
the characters 
``p''
and 
``l''.
A 
``p''
or 
``l''
must be preceded by a blank or tab.
In this case, the command abbreviated by these characters
is executed after the command completes.
Since ex normally prints the new current 
line after each change, 
.B p
is rarely necessary.
Any number of plus (+) or minus (\-) characters may also 
be given with these flags.
If they appear, the specified offset 
is applied to the current line
value before the printing command is executed.
.P
Most commands that change the contents of the editor buffer give
feedback if the scope of the change exceeds a threshold given by the
.B report option.
This feedback helps to detect 
undesirably large changes so that they may
be quickly and easily reversed with the undo
command.
After commands with global effect,
you will be informed if the net change in the number of lines
in the buffer during this command exceeds this threshold.
.SS "\fICommand Addressing\fR"
The following specifies the line addressing  syntax
for ex commands:
.TP 15n
\fB\&.\fR
The current line.
Most commands leave the current line 
as the last line which they affect.
The default address for most commands 
is the current line,
thus ``\fB.\fR''
is rarely used alone as an address.
.TP 
\fIn\fR
The 
.IR n th 
line in the editor's buffer, 
lines being numbered
sequentially from 1.
.TP 
\fB$\fR
The last line in the buffer.
.TP 
\fB%\fR
An abbreviation for 
``1,$'',
the entire buffer.
.TP 
\fI+n\fR\ or \fI\-n\fR
An offset, 
.I n
relative to the current buffer line.
The forms 
``.+3'' ``+3''
and 
``+++''
are all equivalent.
If the current line is line 100 they all address line 103.
.TP 
\fB/\fIpattern\fR\fB/\fR\ or \fB?\fIpattern\fR\fB?\fR
.br
Scan forward and backward respectively for a text matching
the regular expression given by
.IR pattern .
Scans normally wrap around the end of the buffer.
If all that is desired is to print the next 
line containing 
.IR pattern , 
then the trailing slash (/)
or question mark (?)
may be omitted.
If 
.IR pattern
is omitted or explicitly empty, then the string
matching the last specified regular expression is located.
The forms 
``\s-1RETURN\s0''
and 
``?\s-1RETURN\s0''
scan using the last named regular expression. 
After a substitute,
``\s-1RETURN\s0''
and 
``??\s-1RETURN\s0''
would scan using that substitute's regular expression.
.TP 
\fB\'\'\fR\ or \fB\'\fP\fIx\fR
Before each nonrelative motion of the current line dot (.),
the previous current line is marked with a label, 
subsequently referred to with two single quotation
marks (\'\').
This makes it easy to refer or return to this previous context.
Marks are established with the vi
.B m
command, using a single lowercase letter
as the name of the mark.
Marked lines are later
referred to with the notation 
.sp
	\'\fIx\fR.
.sp
where
.I x
is the name of a mark.
.P
Addresses to commands consist of a series of addresses,
separated by a colon (,) or a semicolon (;).
Such address lists are evaluated left to right.
When addresses are separated by 
a semicolon (;) the current line (\|.\|)
is set to the value of the previous addressing expression
before the next address is interpreted.
If more addresses are given than the command requires,
then all but the last one or two are ignored.
If the command takes two addresses, the first addressed line must
precede the second in the buffer.
Null address specifications are permitted in a list of addresses,
the default in this case is the current line 
``.'';
thus 
``,100''
is equivalent to 
``\fB.\fR,100''.
It is an error to give a prefix address 
to a command which expects none.
.SS "\fICommand Format\fR"
The following is the format for all ex commands:
.in +5
.sp
[\fIaddress\fP] [\fIcommand\fP] [!] [\fIparameters\fP] [\fIcount\fP] [\fIflags\fP]
.in -5
.PP
All parts are optional depending on the particular
command and its options. 
The following section describes specific commands.
.SS "\fIArgument List Commands\fR"
The argument list commands allow you to work
on a set of files,
by remembering the list of filenames
that are specified when you invoke vi.
The 
.B args
command lets you examine this list of filenames.
The 
.B file
command gives you information about the current file.
The 
.B n
(next)
command lets you either edit the next file in the argument
list or change the list.
And the 
.B rewind
command lets you restart editing the files in the list.
All of these commands are described below:
.TP 14n
args
The members of the argument list are printed, 
with the current argument delimited by brackets.
For example, a list might look like this:
.sp
	file1 file2 [file3] file4 file5
.sp
The current file is 
\fIfile3\fR.
.TP 14n 
f
Prints the current filename,
whether it has been modified since the last
.B write 
command,
whether it is readonly,
the current linenumber,
the number of lines in the buffer,
and the percentage of the
buffer that you have edited.
In the rare case that the current file is 
``[Not edited]''
this is noted also; 
in this case you have to use the form 
``w!''
to write to
the file, since the editor is not sure that a 
.B w
command
will not destroy a file unrelated 
to the current contents of the buffer.
.TP 14n
f \fIfile\fR
The current filename is changed to
.I file
which is considered 
``[Not edited]''.
.TP 
n
The next file in the command line argument list is edited.
.TP 
n!
This variant suppresses warnings 
about the modifications to the buffer not
having been written out, discarding irretrievably any changes 
that may have been made.
.TP 
n \fB[+\fIcommand\fR\fB]\fR \fIfilelist\fR
The specified
.I filelist
is expanded and the resulting list replaces the
current argument list;
the first file in the new list is then edited.
If
.I command
is given (it must contain no spaces), then it is executed after 
editing the first such file.
.TP 
rew
The argument list is rewound, and the 
first file in the list is edited.
.TP 
rew!
Rewinds the argument list discarding 
any changes made to the current buffer.
.SS "\fIEdit Commands\fR"
To edit a file other than the one you are currently
editing, you will often use one of the variations of
the 
.B e
command.
.P
In the following discussions, note that
the name of the current file is always remembered by
vi and is specified by a percent sign (%).
The name of the
.I previous
file in the editing buffer is specified by a number sign (#).
.P
The edit commands are described below:
.TP 10
e \fIfile\fR
Used to begin an editing session on a new file.
The editor first checks to see 
if the buffer has been modified since the last
.B w
command was issued.
If it has been, a warning is issued and the command is aborted.
The command otherwise deletes the entire contents of the editor buffer,
makes the named file the current file and prints the new filename.
After ensuring that this file is sensible,
(i.e., that it is not a binary file, directory, or a
device),
the editor reads the file into its buffer.
If the read of the file completes without error,
the number of lines and characters read is printed
on the status line.
If there were any non\-ASCII characters
in the file they are stripped of their non\-ASCII high bits,
and any null characters in the file are discarded.
If none of these errors occurred, the file is considered edited.
If the last line of the input file is missing the trailing
newline character, it is supplied and a complaint issued.
The current line is initially the first line of the file.
.TP 
e! \fIfile\fR
This variant form suppresses the complaint about modifications having
been made and not written from the editor buffer, thus
discarding all changes that have been made 
before editing the new file.
.TP 
e \fI+n file\fR
.br
Causes the editor to begin editing at line
.I n
rather than at the first line.
The argument \fIn\fR may also be an editor command 
containing no spaces; for example, 
``+/pattern''.
.TP 
\s-1CNTRL-^\s0
.br
This is a shorthand equivalent for 
``:e\0#\s-1RETURN\s0'', 
which returns to the previous position in the last edited file.
If you do not want to write the file you should
use 
``:e!\ #\s-1RETURN\s0''
instead.
.SS "\fIWrite Commands\fR"
The write commands let you write out all or part of
your editing buffer to either the current file or to
some other file.
These commands are described below:
.TP 
w \fIfile\fR
Writes changes made back to 
.IR file , 
printing the number of lines and characters written.
Normally, 
.I file
is omitted and the buffer is written to the name of
the current file.
If 
.I file
is specified, then text will be written to that file.
The editor writes to a file only 
if it is the current file and is edited,
or if the file does not exist.
Otherwise, you must give the variant form 
.B w!
to force the write.
If the file does not exist it is created.
The current filename is changed only 
if there is no current filename; 
the current line is never changed.
.sp
If an error occurs while writing the current and
.I edited
file, the editor
prints
.sp
	No write since last change
.sp
even if the buffer had not previously been modified.
.TP 
w>> \fIfile\fR
Appends the buffer contents at the end of
an existing file.
Previous file contents are not destroyed.
.TP
w! \fIname\fR
Overrides the checking of the normal 
.B write
command, and writes to any file that the system permits.
.TP  
w \fB!\fIcommand\fR
.br
Writes the specified lines into 
.IR command .
Note the difference between 
.sp
	w! \fIfile\fP
.sp
which overrides checks and
.sp
	w !\fIcmd\fP
.sp
which writes to a command.
The output of this command is displayed on the
screen and not inserted in the editing buffer.
.SS "\fIRead Commands\fR"
The read commands let you read text into your
editing buffer at any location you specify.
The text you read in must be at least one line
long, and can be either a file or the output from a
command.
.TP 10
r \fIfile\fR
Places a copy of the text of the given file in the
editing buffer after the specified line.
If no file is given then the current filename is used.
The current filename is not changed unless 
there is none, in which
case the file becomes the current name.
If the file buffer is empty and there is no current name then
this is treated as an
.B e
command.
.sp
Address 0 is legal for this command 
and causes the file to be read at
the beginning of the buffer.
Statistics are given as for the 
.B e
command when the 
.B r
successfully terminates.
After an
.B r
the current line is the last line read.
.TP 
r \fB!\fIcommand\fR
.br
Reads the output of 
.I command
into the buffer after the specified line.
A blank or tab before the exclamation mark (!) is mandatory.
.SS "\fIQuit Commands\fR"
There are several ways to exit vi.
Some abort the editing session, some write out the
editing buffer before exiting, and some warn you if
you decide to exit without writing out the buffer.
All of these ways of exiting are described below:
.TP 10
q
Exits vi.
No automatic write of the editor buffer to a file is performed.
However, vi issues a warning message 
if the file has changed since the last
.B w
command was issued, and does not
quit.
Vi will also issue a diagnostic 
if there are more files in the argument list left to
edit.
Normally, you will wish to save your changes, and you 
should give a 
.B w
command.
If you wish to discard them, use the 
``q!''
command variant.
.TP 
q!
Quits from the editor, discarding changes 
to the buffer without complaint.
.TP 
wq \fIname\fR
Like a 
.B w
and then a 
.B q
command.
.TP 
wq! \fIname\fR
This variant overrides checking of the 
.B w
command so that you can write to file that the
system allows.
.TP 
x \fIname\fR
If any changes have been made and not written, 
writes the buffer out and then quits.
Otherwise, it just quits.
.SS "\fIGlobal and Substitute Commands\fR"
The global and substitute commands allow you to
perform complex changes to a file in a single command.
Learning how to use these commands is a must
for the serious user of vi.
.TP 10n 
g/\fIpattern\fB/\fIcmds 
.br
The 
.B g
command has two distinct phases.
In the first phase, 
each line matching 
.I pattern
in the editing buffer is marked.
Next, the given command list is executed with the
current line,
dot (\|.\|),
initially set to each marked line.
.sp
The command list consists of the remaining commands on the current
input line and may continue to multiple lines by ending all but the
last such line with a backslash (\|\e\|).
This multiple-line option will not work from within
vi, you must switch to ex to do it.
If
.I cmds
(or the trailing slash (/) delimiter) 
is omitted, then each line matching
.I pattern
is printed.
.sp
The
.B g
command itself may not appear in
.IR cmds .
The options
.I autoprint
and
.I autoindent
are inhibited during a
global command
and the value of the
.I report
option is temporarily infinite,
in deference to a 
.I report
for the entire global.
Finally, the context mark (\|\'\|) or (\|\`\|) is set 
to the value of the current line (\|.\|)
before the global command begins 
and is not changed during a global command.
.sp
The following global commands, most of them
substitutions, cover the most frequent uses of 
the global command.
.TP 15
g/s1/p
This command simply prints all lines that
contain the string 
``s1'' .
.TP 
g/s1/s//s2/
This command substitutes the 
.I first
occurrence of 
``s1''
on all lines that contain it
with the string 
``s2''.
.TP 
g/s1/s//s2/g
This command substitutes all occurrences 
of 
``s1''
with the string 
``s2''.
This includes multiple occurrences of 
``s1'' on a line. 
.TP 
g/s1/s//s2/gp
This command works the same as the preceding example,
except that in addition, all changed lines are
printed on the screen.
.TP 
g/s1/s//s2/gc
This command asks you to confirm that you want to
make each
substitution of the string 
``s1'' with
the string 
``s2''.
If you type a 
``y''
then the given substitution is made,
otherwise it is not.
.TP 
g/s0/s/s1/s2/g
This command marks all those lines that contain the
string 
``s0'',
and then for those lines only, it 
substitutes all occurrences of the string 
``s1'' with 
``s2''.
.TP 
g!/\fIpattern\fB/\fIcmds
This variant form of 
.I g
runs 
.I cmds
at each line not matching 
.IR pattern .
.TP 
s/\fIpattern\fB/\fIrepl\fB/\fIoptions
On each specified line, the first instance of text
matching the regular expression
.I pattern
is replaced by the replacement text
.IR repl .
If the
.I global
indicator option character 
``g'' appears, then all instances on a line are substituted.
If the
.I confirm
indication character 
``c''
appears,
then before each substitution the line to be substituted
is printed on the screen with the string to be substituted marked
with caret (\|^\|) characters.
By typing a 
``y'',
you cause the substitution to be performed;
any other input causes no change to take place.
After an
.B s
command
the current line is the last line substituted.
.br
.ne 1i
.TP 
v/\fIpattern\fB/\fIcmds
A synonym for the
.B global
command variant 
.BR g! , 
running the specified 
.I cmds 
on each line that does not match 
.IR pattern .
.SS "\fIText Movement Commands\fR"
The text movement commands are largely superseded by
commands available in vi command mode.
However, the following two commands are still quite useful.
.TP 15
co addr \fIflags\fR
A copy of the specified lines is placed after
.IR addr ,
which may be 
``0'' .
The current line
``\fB.\fR''
addresses the last line of the copy.
.TP 
[\fIrange\fB]m\fIaddr\fR 
The
.B m
command moves
the lines specified by
.I range
after the line given by
.IR addr .
For example,
``m+''
swaps the current line and the following line,
since the default range is just the current line.
The first of the moved lines becomes the current line (dot).
.SS "\fIShell Escape Commands\fR"
You will often want to escape from the editor to
execute normal \*(x1 commands. 
You may also want to change your working directory
so that your editing can be done with respect to a
different working directory.
These operations are described below:
.TP 15
cd directory
The specified 
.I directory
becomes the current directory.
If no directory is specified, the current value of the
.I home
option is used as the target directory.
After a
.B cd
the current file is not considered to have been
edited so that write restrictions on preexisting files 
still apply.
.TP 
sh
A new shell is created.
You  may invoke as many commands as you like in this shell.
To return to vi, type a 
\s-1CNTRL-D\s0
to terminate the shell.
.TP 
!\fIcommand\fI
The remainder of the line after the exclamation (!) 
is sent to a shell to be executed.
Within the text of
.I command
the characters 
``%''
and 
``#''
are expanded as the filenames 
of the current file and the last edited file and the character
``!''
is replaced with the text of the previous command.
Thus, in particular,
``!!''
repeats the last such shell escape.
If any such expansion is performed, the expanded line is echoed.
The current line is unchanged by this command.
.P
If there has been 
``[No\ write]''
of the buffer contents 
since the last
change to the editing buffer, then a diagnostic is printed
before the command is executed as a warning.
A single exclamation (!) is printed when the command completes.
.SS "\fIOther Commands\fR"
The following command descriptions explain how to
use miscellaneous ex commands that do not fit into
the above categories:
.TP 10
abbr
Maps the first argument to the following string. For
example, the following command
.sp
	:abbr rainbow yellow green blue red
.sp
maps ``rainbow'' to ``yellow green blue red''.
Abbreviations can be turned off with the 
.B unabbreviate 
command, as in:
.sp
	:una rainbow
.TP 10
map, map!
Maps any character or escape sequence to an existing
command sequence. Characters mapped with 
.B map!
work only in insert mode, while characters mapped with 
.B map
work only in command mode.
.TP 10
nu
Prints each specified line preceded by its buffer line number.
The current line is left at the last line printed.
To get automatic line numbering of lines in the
buffer, set the 
.I number 
option.
.TP 
preserve
The current editor buffer is saved 
as though the system had just crashed.
This command is for use only in emergencies when a
.B w
command has resulted in an error 
and you don't know how to save your work.
.TP 
=
Prints the line number of the
addressed line.
The current line is unchanged.
.TP 
recover \fIfile\fR
.br
Recovers  
.I file
from the system save area.
The system saves a copy of the 
editing buffer only if you
have made changes to the file,
the system crashes, or
you execute a
.B preserve
command.
Except when you use
.B preserve
you will be notified by mail when a file is saved.
.TP 
set \fIargument\fR
With no arguments, 
.B set
prints those options whose values have been
changed from their defaults;
with the argument
.I all
it prints all of the option values.
.P
Giving an option name followed by a question mark (?)
causes the current value of that option to be printed.
The 
``?''
is unnecessary unless the option is Boolean valued.
Switch options are given values either with
.sp
	set \fIoption\fR
.sp
to turn them on or
.sp
	set \fInooption\fR
.sp
to turn them off.
String and numeric options are assigned with
.sp
	set \fIoption\fR=value
.sp
More than one parameter may be given to 
.IR set \|;
all are interpreted from left to right.
.TP 
tag \fIlabel\fR
The focus of editing switches to the location of
.IR label .
If necessary, vi will switch to a different file in
the current directory to find 
.IR label .
If you have modified the current file before giving a
.B tag
command, you must first write it out. 
If you give another
.B tag
command with no argument, then the previous
.I label 
is used.
.sp
Similarly, if you type only a
\s-1CNTRL-]\s0,
vi searches for the word immediately after the cursor as a tag.
This is equivalent to typing 
``:tag'',
this word, and then a 
\s-1RETURN\s0.
.sp
The tags file is normally created by a program such as
.B ctags,
and consists of a number of lines 
with three fields separated by blanks or tabs.  
The first field gives the name of the tag,
the second the name of the file where the tag resides, 
and the third gives an addressing 
form which can be used by the editor to find the tag.
This field is usually a contextual scan using 
/\|\fIpattern\fR\|/
to be immune to minor changes in the file.  
Such scans are always performed as if the
.I nomagic
option was set.
The tag names in the tags file must be sorted alphabetically.
There are a number of options that can be set
to affect the vi environment. 
These can be set with the ex
.B set 
command
either while editing 
or immediately after vi is invoked in the 
vi start-up file,
.IR .exrc .
.P
The first thing that must be done 
before you can use vi, is to set the terminal
type so that vi understands how to talk to the 
particular terminal you are using.
.P
Each time vi is invoked, it reads commands from the
file named
.I \&.exrc
in your home directory.
This file normally sets the user's preferred options
so that they need not be set manually each time 
you invoke vi.
Each of the options is described in detail below.
.SS "\fIOptions\fR"
There are only two kinds of options: switch
options and string options.
A switch option is either on or off.
A switch is turned off by prefixing the word
.I no
to the name of the switch within a 
.B set 
command.
String options are strings of characters that
are assigned values with the syntax 
.IR option=string .
Multiple options may be specified on a line.
.I Vi
options are listed below:
.TP
autoindent, ai	default: noai
Can be used to ease the preparation of structured program text.
For each line created by an
append, change, insert, open,
or substitute operation,
vi looks at the preceding line to determine and
insert an
appropriate amount of indentation.
To back the cursor up to the preceding tab stop, you can type
\s-1CNTRL-D\s0.
The tab stops going backward 
are defined as multiples of the
.I shiftwidth
option.
You cannot backspace over the indent,
except by typing a
\s-1CNTRL-D\s0.
.sp
Specially processed in this mode is a line 
with no characters added
to it, which turns into a completely blank line (the whitespace 
provided for the
.I autoindent
is discarded.)
Also specially processed in this mode are lines beginning with
a caret (\|^\|) and immediately followed by a 
\s-1CNTRL-D\s0.
This causes the input 
to be repositioned at the beginning of the line,
but retains the previous indent for the next line.
Similarly, a ``0''
followed by a 
\s-1CNTRL-D\s0
repositions the cursor at the beginning but without
retaining the previous indent.
.I Autoindent
doesn't happen in global
commands.
.TP 
autoprint ap 	default: ap
.br
Causes the current line to be printed after each
ex copy, move, or substitute command.
This has the same effect as supplying a trailing 
``p'' to each such command.
.I Autoprint
is suppressed in globals,
and only applies to the last of many commands on a line.
.TP 
autowrite, aw	default: noaw
.br
Causes the contents of the buffer to be automatically
written to the current file if you have modified it when
you give a
.BR next ,
.BR rewind ,
.BR tag ,
or
.B !
command, or a 
\s-1CNTRL-^\s0
(switch files) 
or 
\s-1CNTRL-]\s0
(tag go to) command.
.TP 
beautify, bf	 default: nobeautify
.br
Causes all control characters except tab, new line and formfeed
to be discarded from the input.
A complaint is registered the first time a
backspace character is discarded.
.I Beautify
does not apply to command input.
.TP 
directory, dir	 default: dir=/tmp
.br
Specifies the directory in which vi
places the editing buffer file.
If this directory is not
writable, then the editor will exit abruptly when it fails 
to write to the buffer file.
.TP 
edcompatible	 default: noedcompatible
.br
Causes the presence or absence of
.B g
and
.B c
suffixes on substitute commands to be remembered, 
and to be toggled on and off
by repeating the suffixes.  
The suffix
.B r
causes the substitution to be like the
.tr ~~
.B ~
.tr ~ 
command, instead of like
.B &.
.TP
errorbells,eb	default: noeb
Error messages are preceded by a bell. If possible, the
editor always places the error message in inverse video
instead of ringing the bell.
.TP 
hardtabs, ht	 default: ht=8
.br
Gives the boundaries on which terminal hardware tabs are set or
on which the system expands tabs.
.TP 
ignorecase, ic	 default: noic
.br
Maps all uppercase characters in the text to 
lowercase in regular expression matching.
In addition, all uppercase characters 
in regular expressions are mapped
to lowercase except in character class specifications
enclosed in brackets.
.TP 
lisp 	default: nolisp
.br
.I Autoindent
indents appropriately for
LISP code, and the \fB( ) { } [[\fR and \fB]]\fR commands 
are modified to have meaning for LISP.
.TP 
list	 default: nolist
.br
All printed lines will be displayed unambiguously,
showing tabs and end-of-lines.
.TP 
magic 	default: magic
.br
If
.I nomagic
is set, the number of regular expression metacharacters is 
greatly reduced, 
with only up-arrow (^) and  dollar sign ($) having special effects.
.tr ~~
In addition the metacharacters 
``~'' and ``&'' 
in replacement patterns are treated as normal characters.
All the normal metacharacters may be made
.tr ~ 
.I magic
when
.I nomagic
is set by preceding them with a backslash (\e).
.TP 
mesg 	default: nomesg
.br
Causes write permission to be turned off to the terminal
while you are in visual mode, if
.I nomesg
is set.
This prevents people writing to your screen with the \*(x1 
.B write 
command and scrambling your screen as you edit.
.TP 
number, n	 default: nonumber
Causes all output lines to be printed with their line numbers.
.TP
open	default:open
If set to \fInoopen\fR, the commands 
.B open
and 
.B visual
are not permitted from ex. This is set to prevent
confusion resulting from accidental entry to open or
visual mode. 
.TP 
optimize, opt	 default: optimize
.br
Output of text to the screen is expedited 
by setting the terminal
so that it does not perform automatic carriage returns
when printing more than one line of output,
thus greatly speeding output on terminals without addressable
cursors when text with leading whitespace is printed.
.TP 
paragraphs, para	 default: para=IPLPPPQPP\0TPbp
.br
Specifies paragraph delimiters 
for the \fB{\fR and \fB}\fR operations.
The pairs of characters in the option's value are the names
of the nroff macros that start paragraphs.
.TP
prompt	default: prompt
Ex input is prompted for with a colon (:). If
\fInoprompt\fR is set, when ex command mode is entered
with the Q command, no colon prompt is displayed on the
status line.
.TP 
redraw 	default: noredraw
.br
The editor simulates (using great amounts of output), 
an intelligent
terminal on a dumb terminal.
Useful only at very high speed.
.TP
remap	default: remap
If on, mapped characters are repeatedly tried until they
are unchanged. For example, if \fIo\fR is mapped to
\fIO\fR and \fIO\fR is mapped to \fII\fR, \fIo\fR will map
to \fII\fR if remap is set, and to \fIO\fR if
\fInoremap\fR is set. 
.TP 
report 	default: report=5
.br
Specifies a threshold for feedback from commands.
Any command that modifies more than the specified number of lines
will provide feedback as to the scope of its changes.
For global commands and the undo command,
which have potentially far reaching scope,
the net change in the number of lines in the buffer is
presented at the end of the command, subject to this same threshold.
Thus notification is suppressed during a
.B g
command on the individual commands performed.
.TP 
scroll 	default: scroll=\(12 window
.br
Determines the number of logical lines scrolled when
\s-1CNTRL-D\s0
is received from a terminal input in command mode,
and the number of lines printed by a command mode
.B z
command (double the value of
.IR  scroll ).
.TP 
sections 	default: sections=SHNHH\0HU
.br
Specifies the section macros for the \fB[[\fR and \fB]]\fR operations.
The pairs of characters in the option's value are the names
of the nroff macros that start paragraphs.
.TP 
shell, sh 	default: sh=/bin/sh
.br
Gives the pathname of the shell forked for 
the shell escape command 
``!'' ,
and by the
.B shell
command.
The default is taken from SHELL in the environment, if present.
.TP 
shiftwidth, sw 	default: sw=8
.br
Gives the width of a software tab stop,
used in reverse tabbing with 
\s-1CNTRL-D\s0
when using
.I autoindent
to append text,
and by the shift commands.
.TP 
showmatch, sm	 default: nosm
.br
When a \fB)\fR or \fB}\fR is typed, 
moves the cursor to the matching
\fB(\fR or \fB{\fR for one second 
if this matching character is on the screen.  
.TP 
tabstop, ts 	default: ts=8
.br
The editor expands tabs in the input file to be on
.I tabstop
boundaries for the purposes of display.
.TP 
taglength, tl	 default: tl=0
.br
The first 
.I taglength 
characters in a tag name are significant,
but all others are ignored.
A value of zero (the default) means 
that all characters are significant.
.TP 
tags	 default: tags=tags /usr/lib/tags
.br
A path of files to be used as tag files for the
.B tag
command.
A requested tag is searched for in the specified files, sequentially.
By default files named
.I tag
are searched for in the current directory and in /usr/lib. 
.TP 
term	 default=value of shell TERM variable
.br
The terminal type of the output device.
.TP 
terse	 default: noterse
.br
Shorter error diagnostics are produced for the experienced user.
.TP 
warn    	default: warn
.br
Warn if there has been 
``[No write since last change]''
before a shell escape command (!).
.TP 
window	 default: window=speed dependent
.br
This specifies the number of lines in a text window.
The default is 8 at slow speeds (600 baud or less),
16 at medium speed (1200 baud),
and the full screen (minus one line) at higher speeds.
.TP 
w300, w1200, w9600
.br
These are not true options but set
.I window
(above)
only if the speed is slow (300), medium (1200), or high (9600),
respectively.
.TP 
wrapscan, ws	 default: ws
.br
Searches using the regular expressions in addressing
will wrap around past the end of the file.
.TP 
wrapmargin, wm	 default: wm=0
.br
Defines the margin for automatic insertion
of newlines during text input.
A value of zero specifies no wrap margin.
.TP 
writeany, wa	  default: nowa
.br
Inhibits the checks normally made before
.B write
commands, allowing a write to any file that the system protection
mechanism will allow.
.SH "Regular Expressions"
A regular expression specifies a set of strings of characters.
A member of this set of strings is said to be \fImatched\fR
by the regular expression.
Vi remembers two previous regular expressions:
the previous regular expression used in a
substitute
command and the previous regular expression used elsewhere,
referred to as the previous 
.I scanning
regular expression.
The previous regular expression
can always be referred 
to by a null regular expression:
e.g., 
``//''
or 
``??''.
.P
The regular expressions allowed by
vi are constructed in one of two ways 
depending on the setting of the
.I magic
option.
The ex and vi default setting of
.I magic
gives quick access to a powerful set of regular expression
metacharacters.
The disadvantage of
.I magic
is that the user must remember that these metacharacters are
.I magic
and precede them with the backslash (\e) 
to use them as 
``ordinary''
characters.
With
.I nomagic
set,
regular expressions are much simpler,
there being only two metacharacters.
The power of the other metacharacters is still available by preceding
the now ordinary character with a 
``\e''.
Note that 
``\e''
is thus always a metacharacter.
In this discussion the magic option is assumed. 
With
.I nomagic
the only special characters are 
the caret (^)
at the beginning
of a regular expression,
the dollar sign ($) at the end of a regular expression,
and the backslash (\e).
.tr ~~
The tilde (\|~\|) and the ampersand (&) also lose 
their special meanings
related to the replacement pattern of a substitute.
.P
The following basic constructs are used to construct
.I magic
mode regular expressions.
.TP 
\fIchar\fR
An ordinary character matches itself.
Ordinary characters are any characters except
a caret (\|^\|) at the beginning of a line,
a dollar sign ($) at the end of line,
a star (*) as any character other than the first,
and any of the following characters:
.sp
	\&.  \e  [  ~
.sp
These characters
must be escaped (i.e., preceded) by a backslash (\e)
if they are to be treated as ordinary characters.
.tr ~ 
.TP 
\fB^\fR
At the beginning of a pattern
this forces the match to succeed only at the beginning of a line.
.TP 
\fB$\fR
At the end of a regular expression this forces the match to
succeed only at the end of the line.
.TP 
\&\fB.\fR
Matches any single character except
the newline character.
.TP 
\fB\e<\fR
Forces the match
to occur only at the beginning of a 
``word''; 
that is, either at the beginning of a line, or just before
a letter, digit, or underline and after a character not one of
these.
.TP 
\fB\e>\fR
Similar to 
``\e<'',
but matching the end of a 
``word'', 
i.e. either the end of the line or before a character
which is not a letter, a digit, or the underline character.
.TP 
\fB[\fIstring\fB]\fR
Matches any single character in the class defined by
.IR string .
Most characters in
.I string
define themselves.
A pair of characters separated by a dash (\-) in
.I string
defines the set of characters between 
the specified lower and upper
bounds, thus 
``[a\-z]''
as a regular expression matches
any single lowercase letter.
If the first character of
.I string
is a caret (\|^\|) then the construct
matches those characters which it otherwise would not.
Thus 
``[^a\-z]''
matches anything but a lowercase letter 
or a newline.
To place any of the characters caret, left bracket,
or dash in
.I string
they must be escaped with a preceding backslash (\e).
.P
The concatenation of two regular expressions first matches 
the leftmost regular expression 
and then the longest string that can be recognized
as a regular expression.
The first part of this new regular expression matches the 
first regular expression and the second part matches the second.
Any of the single character matching regular expressions mentioned
above may be followed by a
``star''
(*) to form a regular expression
that matches zero or more 
adjacent occurrences of the characters
matched by the prefixing regular expression.
.tr ~~
The tilde (~) may be used in a regular expression
to match the text that defined the replacement part
of the last
.B s
command.
A regular expression may be enclosed between the sequences
``\e(''
and 
``\e)''
to remember the text matched by the enclosed regular
expression.
This text can later be interpolated into the
replacement text using the notation
.sp
	\e\fIdigit\fR
.sp
where 
.I digit 
enumerates the set of remembered regular expressions.
.P
The basic metacharacters for the replacement pattern are
the ampersand (&) and the tilde (~)
these are given as 
``\e&''
and 
``\e~''
when
.I nomagic
is set.
Each instance of the ampersand is replaced by the characters
matched by the regular expression.
In the replacement pattern the tilde stands
for the text of the previous replacement pattern.
.P
Other metasequences possible in the replacement pattern
are always introduced by a backslash (\e).
The sequence 
``\e\fIn\fR''
is replaced by the text matched
by the \fIn\fRth regular subexpression enclosed between
``\e(''
and 
``\e)''.
When nested, parenthesized subexpressions are present,
\fIn\fR is determined by counting occurrences of 
``\e(''
starting from the left.
The sequences 
``\eu''
and 
``\el''
cause the immediately 
following character in
the replacement to be converted to uppercase or lowercase, 
respectively, if this character is a letter.
The sequences 
``\eU''
and 
``\eL''
turn such 
conversion on, either until
``\eE''
or 
``\ee''
is encountered, 
or until the end of the replacement pattern.
.tr ~ 
.br
.ne 1i
.SH "Limitations"
When using vi, you should note the following limits:
.TP
250,000 lines in a file
.TP
1024 characters per line
.TP
256 characters per global command list
.TP
128 characters per filename
.TP
128 characters in the previous inserted and deleted text
.TP
100 characters in a shell escape command
.TP
63 characters in a string valued option
.TP
30 characters in a tag name
.SH Notes
The 
.I /usr\|/\|lib\|/\|ex2.13preserve
program can be used to restore vi buffer files that
were lost as a result of a system crash.
The
program
searches the
.B /tmp 
directory for 
vi buffer files and
places them in the directory 
\fB/usr/preserve\fR. The owner can retrieve these
files using the 
.B \-r
option.
.PP
The 
.I /\|usr\|/\|lib\|/\|ex2.13preserve
program must be placed in 
the system startup file,
.BR /etc/rc ,
before the command that cleans out the
.B /tmp
directory.
See the 
.SM XENIX
\fIOperations Guide\fR for more information on 
.BR /etc/rc .
