.\" bm# 0000002440
.\" typ N
.\" ver 68k.generic
.\" mod generic
.\"
.\"     SCCSID(@(#)ld.CP	1.1);


.ds mc 68000
.ds rv Revision 1
.TH LD CP 
.SH Name
ld \- link editor
.SH Syntax
.B ld
[ option ] file ...
.SH Description
.I Ld
combines several
object programs into one, resolves external
references, and searches libraries.
.I Ld 
combines the given object
.I files,
producing an object module which can either be executed or
become the input for a further
.I ld
run.
(In the latter case, the
.B \-r
option must be given
to preserve the relocation records.)
The output of
.I ld
is left by default in the file
.BR a.out .
This file is made executable
only if no errors occurred.
.PP
The files given as arguments 
are concatenated in the order specified.  
The default entry point of the output is the 
beginning of the first routine in the first file.
The C compiler,
.I cc ,
calls
.I ld
automatically unless given the 
.B -c
option.
The command line that
.I cc
passes to 
.I ld
is
.PP
.RS
.B ld /lib/crt0.o files \fIcc-options\fP -lc
.RE
.PP
If any argument is a library, it is searched exactly once
at the point it is encountered in the argument list.
Only those routines defining an unresolved external
reference are loaded.
If a routine from a library
refers to another routine in the library,
and the library has not been processed by
.IR ranlib (CP),
the referenced routine must appear after the
referencing routine in the library.
Thus the order of programs within libraries
may be important.
If the first member of a library is named
\&`__.SYMDEF',
then it is understood to be a dictionary for the
library
such
as produced by
.IR ranlib ;
the dictionary is searched
iteratively to satisfy as many references as possible.
.PP
The symbols `\_etext', `\_edata' and `\_end'
(`etext', `edata' and `end' in C)
are reserved, and if referred to,
are set to the first location above the program,
the first location above initialized data,
and the first location above all data, respectively.
It is erroneous to define these symbols.
.PP
If no errors occur and there are no
unresolved external references, then short form
relocation information is attached and the file is made executable.
This short form relocation information is sufficient
to allow the file to be used for another pass of
.I ld ,
to change the text and data base addresses.
At the same time, the 
.B \-n ,
.B \-i ,
or
.B \-F
options can be used to produce different types of
executable files.
.PP
.I Ld
understands several options.
Except for
.BR \-l ,
they should appear before the names of all object
file arguments.
.TP 
.B  \-s
\&`Strip' the output to save space by removing the symbol table
and relocation records. 
Note that stripping impairs the usefulness of the debugger.
This information can also be removed later with
.IR  strip (CP).
.TP
.B \-sr
Do not attach the short form of relocation.
This does
.I not
imply removing the symbol table, as with
.B \-s .
.TP 
.B  \-u
Take the following argument as a symbol and enter
it as undefined in the symbol table.  This is useful
for loading wholly from a library, since initially the symbol
table is empty and an unresolved reference is needed
to force the loading of the first routine.
.TP 
.B  \-U
Discard all symbols except those that are undefined external.
.TP 
.B  \-g
The same as
.BR \-U ,
except also retain the following list of global symbols.
The list consists of the next command line arguments
and is terminated by the end of the command line,
by
.B \-
alone,
or by any further option beginning with a
.BR \- .
.TP 
.B  \-G
The same as
.BR \-g ,
except that the list of global symbols is taken from the file
named by the following argument.
If the next argument is
.B \- 
alone, the standard input is read.
The symbols may be separated by any type of whitespace.
.TP 
.BI \-l x
This
option is an abbreviation for the library name
.RI `/lib/lib x .a',
where
.I x
is a string.
If the library does not exist,
.I ld
then tries
.RI `/usr/lib/lib x .a'.
A library is searched when its name is encountered,
so the placement of a
.B  \-l
is significant.
Note that 
.B \-l
with no argument, defaults to 
.B \-lc .
If the processor on which 
.B ld
is running is not the same as the target processor,
then it is possible that 
.B \-p
may be implied.
In the case of the MC68000 target,
.B "\-p /usr/lib/mlib"
is implied.
.TP
.B \-p
Take the following argument as the directory in which 
.B -l\fIx\fP
libraries will be found.
.TP 
.B  \-x
Do not preserve local
(non.globl) symbols in the output symbol table; only enter
external symbols.
This option saves some space in the output file.
.TP 
.B  \-X
Save local symbols
except for those whose names begin with `L'.
This option is used by
.IR cc (CP)
to discard internally generated labels while
retaining symbols local to routines.
.TP 
.B  \-r
Generate (long form) relocation records in the output file
so that the output file can be the subject of another
.I ld
run.
This flag also prevents final definitions from being
given to common symbols
and suppresses the `undefined symbol' diagnostics.
.TP 
.B  \-d
Force definition of common storage
even if the
.B \-r
flag is present.
.TP 
.B  \-nn
Arrange that when the output file is executed,
the text portion will be read-only and shared
among all users executing the file.
This involves moving the data areas up to the first
possible page boundary following the end of the text.
A warning is issued if the current machine does not
support this option.
.TP 
.B  \-nr
Identical to 
.B \-nn
except that the text and data positions are reversed.
.TP 
.B  \-n
Identical to whichever of 
.B \-nn
and 
.B \-nr 
is the default for the current machine.
.TP 
.B  \-i
When the output file is executed, the program text
and data areas are given separate address spaces.
The only difference between this option and 
.B \-n
is that with 
.B \-i 
the data may start at a boundary unrelated to the
position of the text.
A warning is issued if the current machine does not
support this option.
.TP 
.B  \-o
The
.I name
argument after
.B \-o
is used as the name of the
.I ld
output file, instead of
.BR a.out .
.TP
.B \-e
The following argument is taken to be the
name of the entry point of the loaded
program. 
The base of the text segment is the default.
.TP
.B \-D
The next argument is a decimal number that sets
the size of the data segment.
.TP
.B \-N
The next argument is taken to be a hexadecimal number that
sets the page size, or rounding size, for use with the
.B \-n
option.
With 
.BR \-i ,
it specifies the base of the data segment.
With
.BR \-nn ,
it is used to compute the base of the data segment.
With
.BR \-nr ,
it is used to compute the base of the text segment.
.TP
.B \-R
The next argument is taken to be a hexadecimal number that
is used as the base address for text relocation.
With 
.B \-i
or 
.B \-nn ,
it also specifies the text base address;
with 
.B \-nr
it specifies the data base address.
.TP
.B \-F
The next argument is taken to be a hexadecimal number that
specifies the size of the stack required by the object
file when executing.
This only has meaning on those processors that cannot
expand the stack dynamically.
.SH Files
.ta \w'/usr/lib/lib*.a\ \ 'u
/lib/lib*.a	libraries
.br
/usr/mlib/lib*.a	more libraries
.br
a.out	output file
.SH "See Also"
as(CP), ar(CP), cc(CP), ranlib(CP), strip(CP), a.out(F)
