.\" bm# 0000002248
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)lint.CP	1.1);


.ds mc 
.ds rv Revision 1
.TH LINT CP
.SH Name
lint \- Checks C language usage and syntax.
.SH Syntax
.B lint
.RB [ \-abchlnpuvx ]
file ...
.SH Description
.I Lint\^
attempts to detect features of the C program
.I file\^
that are
likely to be bugs, nonportable, or wasteful.
It also checks type usage more strictly
than the C compiler.
Among the things which are currently detected are
unreachable statements,
loops not entered at the top,
automatic variables declared and not used,
and logical expressions whose value is constant.
Moreover, the usage of functions is checked to find
functions which return values in some places and not in others,
functions called with varying numbers of arguments,
and functions whose values are not used.
.PP
If more than one 
.I file
is given,
it is assumed that all the files
are to be loaded together; they are checked for
mutual compatibility.
If routines from the standard library are called
from
.IR file ,
.I lint\^
checks the function definitions using the standard lint library
.BR llibc.ln .
If
.I lint\^
is invoked with the
.B \-p
option, it checks function definitions from the portable 
lint library
.BR llibport.ln .
.PP
Any number of
.I lint\^
options may be used, in any order.
The following options are used to suppress certain kinds of complaints:
.TP
.B \-a
Suppresses complaints about assignments of long values 
to variables that are not
long.
.TP
.B \-b
Suppresses complaints about
.B break
statements that cannot be reached.
(Programs produced by 
.I lex\^
or
.I yacc\^
will often result in a large number of such complaints.)
.TP
.B \-c
Suppresses complaints about casts that have questionable portability.
.TP
.B \-h
Does not apply heuristic tests that attempt to intuit bugs, improve
style, and reduce waste.
.TP
.B \-u
Suppresses complaints about functions and external variables 
used and not defined,
or defined and not used.
(This option is suitable for running
.I lint\^
on a subset of files of a larger program.)
.TP
.B \-v
Suppresses complaints about unused arguments in functions.
.TP
.B \-x
Does not report variables referred to by external declarations 
but never used.
.PP
The following arguments alter
.I lint's\^
behavior:
.TP
.B \-n
Does not check compatibility against either the standard or the portable
lint library.
.TP
.B \-p
Attempts to check portability to other dialects
of C.
.TP
.BR \-l libname
Checks functions definitions in the specified lint library.
For example, 
.B  "\-lm"
causes the library 
.I llibm.ln
to be checked.
.PP
The
.BR \-D ,
.BR \-U ,
and
.B \-I
options of
.IR cc (CP)
are also recognized as separate arguments.
.PP
Certain conventional comments in the C source
will change the behavior of
.IR lint :
.RS
.TP
/\(**\s-1NOTREACHED\s0\(**/
At appropriate points
stops comments about unreachable code.
.TP
.RI /\(**\s-1VARARGS\s+1 n \(**/
Suppresses
the usual checking for variable numbers of arguments
in the following function declaration.
The data types of the first
.I n\^
arguments are checked;
a missing
.I n\^
is taken to be 0.
.TP
/\(**\s-1ARGSUSED\s0\(**/
Turns on the
.B \-v
option for the next function.
.TP
/\(**\s-1LINTLIBRARY\s0\(**/
Shuts off complaints about
unused functions in this file.
.RE
.PP
.I Lint\^
produces its first output on a per source file basis.
Complaints regarding included files are collected and printed
after all source files have been processed.
Finally,
information gathered from all input files is collected and checked for
consistency.
At this point,
if it is not clear whether a complaint stems from a given 
source file or from
one of its included files,
the source filename will be printed followed by a question mark.
.PP
.SH Files
/usr/lib/lint[12]	Program files
.PP
/usr/lib/llibc.ln, /usr/lib/llibport.ln, /usr/lib/llibm.ln,
/usr/lib/llibdbm.ln, /usr/lib/llibtermlib.ln
.br
	Standard lint libraries (binary format)
.PP
.ne 1i
/usr/lib/llibc, /usr/lib/llibport, /usr/lib/llibm, /usr/lib/llibdbm,
/usr/lib/llibtermlib
.br
	Standard lint libraries (source format)
.PP
/usr/tmp/\(**lint\(**	Temporaries
.PD
.SH "See Also"
cc(CP) 
.SH Notes
.IR Exit (S),
and other functions which do not return,
are not understood. 
This can cause improper error messages.
