.\" bm# 0000002246
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)m4.CP	1.1);


.ds mc 
.ds rv Revision 1
.TH M4 CP
.SH Name
m4 \- Invokes a macro processor.
.SH Syntax
.B m4
[ options ]
[ files ]
.SH Description
.if t .ds ' \h@.05m@\s+4\v@.333m@\'\v@-.333m@\s-4\h@.05m@
.if n .ds ' '
.if t .ds ` \h@.05m@\s+4\v@.333m@\`\v@-.333m@\s-4\h@.05m@
.if n .ds ` `
.I M4\^
is a macro processor
intended as a front end for Ratfor, C, and other languages.
Each of the argument 
.I files 
is processed in order;
if there are no files, or if a filename is
.BR \- ,
the standard input is read.
The processed text is written on the standard output.
.PP
The options and their effects are as follows:
.TP
.B \-e
Operates interactively.
Interrupts are ignored and the output is unbuffered.
.TP
.B \-s
Enables line sync output for the C preprocessor
.RB ( # "line .\|.\|.\|)"
.TP
.BI \-B int\^
Changes the size of the push-back and argument collection
buffers from the default of 4,096.
.TP
.BI \-H int\^
Changes the size of the symbol table hash array from the
default of 199.
The size should be prime.
.TP
.BI \-S int\^
Changes the size of the call stack from the default of 100 slots.
Macros take three slots, and nonmacro arguments take one.
.TP
.BI \-T int\^
Changes the size of the token buffer from the default of 512 bytes.
.PP
To be effective, these flags must appear before any
filenames and before any
.B \-D
or
.B \-U
flags:
.TP
\f3\-D\fP\f2name\^\fP[\f3=\fP\f2val\^\fP]
Defines
.I name\^
to
.I val\^
or to null in
.IR val 's
absence.
.TP
.BI \-U name\^
Undefines
.IR name .
.br
.ne 1i
.SH "Macro Calls"
Macro calls
have the form:
.PP
.RS
name(arg1,arg2, .\|.\|., argn)
.RE
.PP
The
.B (
must immediately follow the name of the macro.
If a defined macro name is not followed by a
.BR ( ,
it is deemed to have no arguments.
Leading unquoted blanks, tabs, and newlines are ignored 
while collecting arguments.
Potential macro names consist of alphabetic letters,
digits, and underscore
.BR _ ,
where the first character is not a digit.
.PP
Left and right single quotation marks are used to quote strings.
The value of a quoted string is the string stripped of the 
quotation marks.
.PP
When a macro name is recognized,
its arguments are collected by searching for a matching right
parenthesis.
Macro evaluation proceeds normally during the collection 
of the arguments,
and any commas or right parentheses
which happen to turn up within the value of a nested
call are as effective as those in the original input text.
After argument collection,
the value of the macro is pushed back onto the input stream
and rescanned.
.PP
.I M4\^
makes available the following built-in macros.
They may be redefined, but once this is done 
the original meaning is lost.
Their values are null unless otherwise stated.
.TP 12
define
The second argument is installed as the value of the macro
whose name is the first argument.
Each occurrence of
.BI $ n\^
in the replacement text,
where
.I n\^
is a digit,
is replaced by the
.IR n -th
argument.
Argument 0 is the name of the macro;
missing arguments are replaced by the null string;
.B $#
is replaced by the number of arguments;
.B $\(**
is replaced by a list of all the arguments separated by commas;
.B $@
is like
.BR $\(** ,
but each argument is quoted (with the current quotation marks).
.TP
undefine
Removes the definition of the macro named in its argument.
.TP
defn
Returns the quoted definition of its argument(s).
It is useful for renaming macros, especially built-ins.
.TP
pushdef
Like
.IR define ,
but saves any previous definition.
.TP
popdef
Removes current definition of its argument(s),
exposing the previous one if any.
.TP
ifdef
If the first argument is defined, the value 
is the second argument, otherwise the third.
If there is no third argument, the value is null.
The word
.SM XENIX
is predefined in
.IR M4 .
.TP
shift
Returns all but its first argument.
The other arguments are quoted and pushed back with
commas in between.
The quoting nullifies the effect of the extra scan that
will subsequently be performed.
.TP
changequote
Changes quotation marks to the first and second arguments.
The symbols may be up to five characters long.
.I Changequote\^
without arguments restores the original values
(i.e., \*`\|\*').
.TP
changecom
Changes left and right comment markers from the default
.B #
and newline.
With no arguments, the comment mechanism is effectively
disabled.
With one argument, the left marker becomes the argument and
the right marker becomes newline.
With two arguments, both markers are affected.
Comment markers may be up to five characters long.
.TP
divert
.I M4\^
maintains 10 output streams,
numbered 0-9.
The final output is the concatenation of the streams
in numerical order;
initially stream 0 is the current stream.
The
.I divert\^
macro changes the current output stream to its (digit-string)
argument.
Output diverted to a stream other than 0 through 9
is discarded.
.TP
undivert
Causes immediate output of text from diversions named as
arguments, or all diversions if no argument.
Text may be undiverted into another diversion.
Undiverting discards the diverted text.
.TP
divnum
Returns the value of the current output stream.
.TP
dnl
Reads and discards characters up to and including the next newline.
.TP
ifelse
Has three or more arguments.
If the first argument is the same string as the second,
then the value is the third argument.
If not, and if there are more than four arguments, 
the process is repeated with arguments 4, 5, 6 and 7.
Otherwise, the value is either the fourth string, 
or if it is not present,
null.
.TP
incr
Returns the value of its argument incremented by 1.
The value of the argument is calculated
by interpreting an initial digit-string as a decimal number.
.TP
decr
Returns the value of its argument decremented by 1.
.TP
eval
Evaluates its argument as an arithmetic 
expression, using 32-bit arithmetic.
Operators include
.BR + ,
.BR \- ,
.BR \(** ,
.BR / ,
.BR % ,
.B ^
(exponentiation),
bitwise
.BR & ,
\(bv,
.BR ^ ,
and
.BR ~ ;
relationals; parentheses.
Octal and hex numbers may be specified as in C.
The second argument specifies the radix for the result;
the default is 10.
The third argument may be used to specify the minimum number
of digits in the result.
.TP
len
Returns the number of characters in its argument.
.TP
index
Returns the position in its first argument 
where the second argument begins (zero origin),
or \-1 if the second argument does not occur.
.TP
substr
Returns a substring of its first argument.
The second argument is a zero origin
number selecting the first character;
the third argument indicates the length of the substring.
A missing third argument is taken to be large enough to extend to
the end of the first string.
.TP
translit
Transliterates the characters in its first argument
from the set given by the second argument to the set given by the third.
No abbreviations are permitted.
.TP
include
Returns the contents of the file named in the argument.
.TP
sinclude
Identical to
.IR include ,
except that it
says nothing if the file is inaccessible.
.TP
syscmd
Executes the
.SM XENIX
command given in the first argument.
No value is returned.
.TP
sysval
Is the return code from the last call to
.IR syscmd .
.TP
maketemp
Fills in a string of
.SM XXXXX
in its argument with the current process
.SM ID\*S.
.TP
m4exit
Causes immediate exit from
.IR m4 .
Argument 1, if given, is the exit code;
the default is 0.
.TP
m4wrap
Argument 1 will be pushed back at final
.SM EOF\*S;
example: m4wrap(\*`cleanup(\|)\*')
.TP
errprint
Prints its argument
on the diagnostic output file.
.TP
dumpdef
Prints current names and definitions,
for the named items, or for all if no arguments are given.
.TP
traceon
With no arguments, turns on tracing for all macros
(including built-ins).
Otherwise, turns on tracing for named macros.
.TP
traceoff
Turns off trace globally and for any macros specified.
Macros specifically traced by
.I traceon\^
can be untraced only by specific calls to
.IR traceoff .
.dt
'\".SH SEE ALSO
'\".I "The M4 Macro Processor\^"
'\"by B. W. Kernighan and D. M. Ritchie.
