.\" bm# 0000002244
.\" typ N
.\" ver generic
.\" mod generic
.\"
.\"     SCCSID(@(#)mkstr.CP	1.1);


.ds mc 
.ds rv Revision 1
.TH MKSTR CP
.SH Name
mkstr \- Creates an error message file from C source.
.SH Syntax
.B mkstr
.RB [ \- ]
messagefile prefix file ...
.SH Description
.I Mkstr
is used to create files of error messages.
Its use can make programs with large numbers of error diagnostics much
smaller, and reduce system overhead in running the program as the
error messages do not have to be constantly swapped in and out.
.PP
.I Mkstr
will process each specified
.IR file ,
placing a massaged version of the input file in a file whose name
consists of the specified
.I prefix 
and the original name.
The optional dash (\-)
causes the error messages to be placed at the end of the specified
message file for recompiling part of a large
.IR mkstr \|ed
program.
.PP
A typical 
.I mkstr
command line is
.DT
.PP
	mkstr pistrings xx *.c
.PP
This command causes all the error messages from the C source
files in the current directory to be placed in the file
.I pistrings
and processed copies of the source for these files to be placed in
files whose names are prefixed with
.I xx.
.PP
To process the error messages in the source to the message file,
.I mkstr
keys on the string
\%`error("'
in the input stream.
Each time it occurs, the C string starting at the `"' is placed
in the message file followed 
by a null character and a newline character;
the null character terminates the message so it can be easily used
when retrieved, the newline character makes it possible to sensibly
.I cat
the error message file to see its contents.
The massaged copy of the input file then contains a
.I lseek
pointer into the file which can be used to retrieve the message. For
example, the command changes
.PP
.RS
error("Error on reading", a2, a3, a4);
.RE
.PP
into
.PP
.RS
error(\fIm\fP, a2, a3, a4);
.RE
.PP
where
.I m
is the seek position of the string in the resulting
error message file.   The programmer must create a
routine
.I error
which opens the message file, reads the string, and
prints it out.  The following example illustrates such a
routine. 
.SH "Example"
.RS
.nf
\fBchar\fR	efilname[] =  "/usr/lib/pi_strings";
\fBint\fR	efil = -1;
.sp
error(a1, a2, a3, a4)
{
	\fBchar\fR buf[256];

	\fBif\fR (efil < 0) {
		efil = open(efilname, 0);
		\fBif\fR (efil < 0) {
			perror(efilname);
			exit(C);
		}
	}
	\fBif\fR (lseek(efil, (long) a1, 0) |\|| read(efil, buf, 256) <= 0)
		\fBgoto\fR oops;
	printf(buf, a2, a3, a4);
}
.fi
.RE
.PP
.SH "See Also"
lseek(S), xstr(CP)
.SH Credit
This utility was developed 
at the University of California at Berkeley 
and is used with permission.
.SH Notes
All the arguments except the name of the file to be processed are
unnecessary.
